/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.client.internal.FeedClient");

dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility");

(function() {

/*
 * Aliases
 */
var URLUtility = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility;

/*
 * Constants
 */
var _SERVICE_URI = "com.ibm.team.dashboard.viewlets.service.internal.feed.IFeedService";

/*
 * Static class for calling feed service operations.
 */
com.ibm.team.dashboard.viewlets.web.client.internal.FeedClient = {

	/*
	 * Retrieves a FeedResponseDTO from the feed at the given url.
	 */
	getFeed: function(/* string */ url, /* string*/ user, /* string */ password, /* string */ id, /* string */ ref, /* number */ startIndex, /* number */ count, /* boolean */ entries, /* boolean */ details, /* boolean */ feedInfo, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var headers = {};
		headers["accept"] = "text/json";
		headers["content-type"] = "application/x-www-form-urlencoded; charset=utf-8";
		var params = "url=" + encodeURIComponent(url);
		if (user) {
			params += "&user=" + encodeURIComponent(user);
		}
		if (password) {
			params += "&password=" + encodeURIComponent(password);
		}
		if (id) {
			// id is used to fetch one specific entry by id
			params += "&id=" + encodeURIComponent(id);
		}
		else {
			if (startIndex && startIndex > 0) {
				params += "&startIndex=" + startIndex;
			}
			if (count) {
				params += "&count=" + count;
			}
		}
		if (ref) {
			/*
			 * Read marks are stored as item indexes relative to a reference point
			 * (the first item for that feed at that time). Because of paging, we
			 * may lose our reference on a different page, so we pass in the old ref
			 * here and get its new index in the response (refIndex).
			 */
			params += "&ref=" + encodeURIComponent(ref);
		}
		if (entries) {
			params += "&entries=true";
		}
		if (details) {
			params += "&details=true";
		}
		if (feedInfo) {
			params += "&feedInfo=true";
		}
		var bindArgs = {
			url: URLUtility.getContextRoot() + "/service/" + _SERVICE_URI + "/feed?" + params,
			handleAs: "json",
			headers: headers,
			preventCache: dojo.isIE,
			handle: function(response, ioArgs){
				if (response instanceof Error) {
					srh.doErrorCallback({
						status: ioArgs.xhr.status,
						message: ioArgs.xhr.statusText
					});
				}
				else {
					srh.doCallback(response);
				}
			}
		};
		var xhr = dojo.xhrGet(bindArgs).ioArgs.xhr;
		if (tracker) {
			tracker.register(xhr, srh);
		}
	},
	
	getJazzFeed: function(/* string[] */ filters, /* string */ processItemId, /* number */processItemScope, /* boolean */ showActive, /* boolean */ scopeSubteams, /* string */ id, /* string */ ref, /* number */ startIndex, /* number */ count, /* boolean */ entries, /* boolean */ details, /* boolean */ feedInfo, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var headers = {};
		headers["accept"] = "text/json";
		headers["content-type"] = "application/x-www-form-urlencoded; charset=utf-8";
		var params = "filters=" + encodeURIComponent(dojo.toJson(filters));
		params += "&processItemId=" + encodeURIComponent(processItemId);
		params += "&processItemScope=" + encodeURIComponent(processItemScope);
		if (showActive) {
			params += "&showActive=" + encodeURIComponent(showActive);
		}
		if (scopeSubteams) {
			params += "&scopeSubteams=" + encodeURIComponent(scopeSubteams);
		}
		if (id) {
			// id is used to fetch one specific entry by id
			params += "&id=" + encodeURIComponent(id);
		}
		else {
			if (startIndex && startIndex > 0) {
				params += "&startIndex=" + startIndex;
			}
			if (count) {
				params += "&count=" + count;
			}
		}
		if (ref) {
			/*
			 * Read marks are stored as item indexes relative to a reference point
			 * (the first item for that feed at that time). Because of paging, we
			 * may lose our reference on a different page, so we pass in the old ref
			 * here and get its new index in the response (refIndex).
			 */
			params += "&ref=" + encodeURIComponent(ref);
		}
		if (entries) {
			params += "&entries=true";
		}
		if (details) {
			params += "&details=true";
		}
		if (feedInfo) {
			params += "&feedInfo=true";
		}
		var bindArgs = {
			url: URLUtility.getContextRoot() + "/service/" + _SERVICE_URI + "/jazzFeed?" + params,
			handleAs: "json",
			headers: headers,
			preventCache: dojo.isIE,
			handle: function(response, ioArgs){
				if (response instanceof Error) {
					serviceResponseHandler.doErrorCallback({
						status: ioArgs.xhr.status,
						message: ioArgs.xhr.statusText
					});
				}
				else {
					srh.doCallback(response);
				}
			}
		};
		var xhr = dojo.xhrGet(bindArgs).ioArgs.xhr;
		if (tracker) {
			tracker.register(xhr, srh);
		}
	}
};

})();
