/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.client.internal.ViewletMemento");

dojo.require("com.ibm.team.dashboard.web.client.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.IMemento");

(function() {

var Constants = com.ibm.team.dashboard.web.client.internal.Constants;
var IMemento = com.ibm.team.dashboard.web.ui.IMemento;

/*
 * A memento that stores viewlet data shared by all users, e.g. content. It is
 * stored in the Viewlet helper of a Dashboard item.
 */
dojo.declare("com.ibm.team.dashboard.web.client.internal.ViewletMemento",
	IMemento, {

	constructor: function(/* object */ memento, /* ViewletHelper */ viewletHelper) {
		this._memento = memento;
		this._viewletHelper = viewletHelper;
		viewletHelper._memento = this;
	},
	
	/*
	 * Serializes the memento into a JSON string.
	 */
	/* string */ serialize: function() {
		return dojo.toJson(this._memento);
	},
	
	startEditing: function() {
		this._workingCopy = dojo.clone(this._memento);
	},
	
	stopEditing: function(/* boolean */ commit, /* boolean */ forcedCommit) {
		if (commit) {
			this._memento = this._workingCopy;
			this._save(forcedCommit);
		}
		delete this._workingCopy;
			
	},

	_save: function(forcedCommit) {
		this._viewletHelper.getDashboardItem().addModification({type:Constants.MODIFICATION_TYPE_SET_VIEWLET_MEMENTO, viewletId:this._viewletHelper.getId(), memento:this.serialize()});
		if (forcedCommit) {
			this._viewletHelper.getDashboardItem().save();
		}
	},
	
	get: function(/* string */ key) {
		var memento = this._workingCopy ? this._workingCopy : this._memento;
		if (memento[key])
			return memento[key];
		return null;
	},
	
	set: function(/* string */ key, /* object OR primative */ value) {
		var memento = this._workingCopy ? this._workingCopy : this._memento;
		memento[key] = value;
		if (!this._workingCopy)
			this._save();
	}
});

})();