/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.MembersViewlet");

dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.PagedTableViewlet");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.MembersClient");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.MemberInfo");
dojo.require("net.jazz.ajax.ui.internal.HoverPopup");
dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.MemberInfoClient");
dojo.require("com.ibm.team.dashboard.web.util.internal.HTMLUtility");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "MembersMessages");

(function() {

var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var PagedTableViewlet = com.ibm.team.dashboard.viewlets.web.ui.internal.PagedTableViewlet;
var MembersClient = com.ibm.team.dashboard.viewlets.web.client.internal.MembersClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var MemberInfo = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.MemberInfo;
var HoverPopup = net.jazz.ajax.ui.internal.HoverPopup;
var HTMLUtility = com.ibm.team.dashboard.web.util.internal.HTMLUtility;
var MemberInfoClient = com.ibm.team.dashboard.viewlets.web.client.internal.MemberInfoClient;
	
var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;
var messages;

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.MembersViewlet",
	PagedTableViewlet, {
	
	constructor: function(args) {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "MembersMessages");
	},

	onInit: function() {
		this._updateTitle();
		this._widgets = new Array();
	},
	
	onRefresh: function() {
		delete this._totalResults;
		delete this._token;
		this._destroyWidgets();
		this._infoWidgets = new Object();
		this._userItemIds = new Array();
	},
	
	getResultCount: function() {
		return (this._totalResults || this._totalResults == 0) ? this._totalResults : -1;
	},
	
	getNoResultsMessage: function() {
		if (!this.getScopeItem()) {
			return this._createNoScopeMsg();
		}
		if (this.getScope() == DashboardConstants.SCOPE_TEAM_AREA)
			return document.createTextNode(messages.noMembersTeam);
		return document.createTextNode(messages.noMembersProject);
	},
	
	getItemsPerPage: function() {
		return this.getPreference("items");
	},
	
	getContentClass: function() {
		return "MembersViewlet";
	},
	
	fetch: function(srh, start, count, entries, details){
		// handle Auto(none) scope when no team areas
		if (!this.getScopeItem()) {
			this._totalResults = 0;
			this._totalResultsChanged();
			this.getSite().setSecondaryTitle("");
			srh.doCallback();
			return;
		}
		if (details && !entries) {
			this.fetchDetails(srh, start, count);
			return;
		}
		var serviceObject = {
			self: this,
			success: function(page) {
				if (page && page.totalResults > 0) {
					if (this.self._totalResults != page.totalResults) {
						this.self._totalResults = page.totalResults;
						this.self._totalResultsChanged();
					}
					this.self._token = page.token;
					if (page.members) {
						var members = page.members;
						var result = new Object();
						for (var i = 0; i < members.length; i++) {
							this.self._userItemIds[i+start] = members[i].itemId;
						}
						if (entries)
							result.entries = members;
						if (details)
							result.details = members;
						srh.doCallback(result);
					}
				} else {
					this.self._totalResults = 0;
					this.self._totalResultsChanged();
					srh.doCallback();
				}
			},
			failure: function(error) {
				srh.doErrorCallback(error);
			}
		}
		var srh2 = new ServiceResponseHandler(serviceObject, "success", "failure");
		var items = this.getPreference("items");
		switch(this.getScope()) {
			case DashboardConstants.SCOPE_TEAM_AREA:
				MembersClient.getMembersForTeamArea(this.getScopeItem().itemId, start, count, this._token ? this._token : "", details, srh2, this.getServiceTracker());
				break;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				MembersClient.getMembersForProjectArea(this.getScopeItem().itemId, start, count, this._token ? this._token : "", details, srh2, this.getServiceTracker());
				break;
		}
	},
	
	fetchDetails: function(srh, start, count) {
		var membersToFetch = new Array();
		var result = new Object();
		result.details = new Array();
		for (var i = start; i < start + count && this._userItemIds[i]; i++) {
			membersToFetch.push(this._userItemIds[i]);
		}
		var serviceObject = {
			self: this,
			success: function(dtos) {
				var j = 0;
				for (var i = 0; i < dtos.length; i++) {
					while (result.details[j]) j++;
					result.details[j] = {details: dtos[i], itemId: this.self._userItemIds[j+start]};
				}
				srh.doCallback(result);
			},
			failure: function(error) {
				srh.doErrorCallback(error);
			}
		}
		var srh2 = new ServiceResponseHandler(serviceObject, "success", "failure");
		MemberInfoClient.getMemberInfo(membersToFetch, srh2, this.getServiceTracker());
	},

	/* Node[] */ renderEntry: function(member, memberDetails, /* boolean */ zoomed) {
		var columns = new Array();
		
		if (this.getSite().canViewDashboard(DashboardConstants.SCOPE_TEAM_AREA)) {
			var href = "#action=" + DashboardConstants.ACTION_VIEW_DASHBOARD + "&" + DashboardConstants.URL_PARAM_USER + "=" + member.userId;
			var node = document.createElement("a");
			node.href = href;
		} else {
			var node = document.createElement("span");
		}
		node.appendChild(document.createTextNode(member.name));
		var nameCell = document.createElement("td");
		nameCell.className = "nameCell";
		var tip = document.createElement("span");
		var tooltip = new HoverPopup({
				connectNode: [node],
				callback: dojo.hitch(this,this._populateInfoTooltip),
				callbackArg: {tipSpan: tip, userItemId: member.itemId},
				repopulateDelay: -1,
				preDisplayCheck: dojo.hitch(this,function(){ return !this.isZoomed(); })
				}, tip);
		this._widgets.push(tooltip);
		nameCell.appendChild(node);
		columns.push(nameCell);
		
		var roleCell = document.createElement("td");
		roleCell.className = "roleCell";
		for (var i = 0; member.roles && i < member.roles.length; i++) {
			var roleSpan = document.createElement("span");
			roleSpan.appendChild(document.createTextNode(member.roles[i]));
			
			var tipSpan = document.createElement("span");
			var innerSpan = document.createElement("span");
			innerSpan.innerHTML = messages.loading;
			innerSpan.className = "com-ibm-team-dashboard-web-viewlets-ui-internal-MembersViewlet-TooltipLoading";
			tipSpan.appendChild(innerSpan);
			tooltip = new HoverPopup({
				connectNode: [roleSpan],
				callback: dojo.hitch(this,this._populateRoleTooltip),
				callbackArg: {tipSpan: innerSpan, roleId: member.roles[i],
					processAreaScope: this.getScope(),
					processAreaItemId: this.getScopeItem().itemId},
				repopulateDelay: -1}, tipSpan);
			this._widgets.push(tooltip);
			
			roleCell.appendChild(roleSpan);
			if (i != member.roles.length - 1)
				roleCell.appendChild(document.createTextNode(", "));
		}
		columns.push(roleCell);
		
		return columns;
	},
	
	/* Node[] */ renderDetails: function(member) {
		if (!this._infoWidgets[member.itemId]) {
			var memberInfo = new MemberInfo({memberInfoDto: member.details, linkTeams: this.getSite().canViewDashboard(DashboardConstants.SCOPE_TEAM_AREA), linkProjects: this.getSite().canViewDashboard(DashboardConstants.SCOPE_PROJECT_AREA)});
			this._widgets.push(memberInfo);
			this._infoWidgets[member.itemId] = memberInfo;
		}
		return this._createDetailsCell(this._infoWidgets[member.itemId]);
	},
	
	_createDetailsCell: function(infoWidget) {
		var columns = new Array();
		
		var cell = document.createElement("td");
		cell.className = "detailsCell";
		cell.colSpan = 2;
		cell.appendChild(infoWidget.domNode);
		columns.push(cell);
		
		return columns;
	},
	
	_createNoScopeMsg: function() {
		var msgSpan = document.createElement("span");
		msgSpan.innerHTML = messages.notScoped;
		if (this.getSite().isDashboardEditing()) {
			var editSpan = document.createElement("span");
			var nodes = I18nUtil.addMessage(editSpan, messages.notScopedEdit)
			this.connect(nodes[0], "onclick", this._handleToggleSettings);
			msgSpan.appendChild(editSpan);
		}
		return msgSpan;
	},
	
	_handleToggleSettings: function(event) {
		dojo.stopEvent(event);
		this.getSite().toggleSettings();
	},

	_populateInfoTooltip: function(args) {
		if (this._infoWidgets[args.userItemId]) {
			args.tipSpan.innerHTML = this._infoWidgets[args.userItemId].domNode.innerHTML;
			return;
		}
		var loadingDiv = document.createElement("div");
		loadingDiv.className = "com-ibm-team-dashboard-web-viewlets-ui-internal-MembersViewlet-TooltipLoading";
		loadingDiv.innerHTML = messages.loading;
		args.tipSpan.appendChild(loadingDiv);
		var serviceObj = {
			self: this,
			success: function(dtos) {
				var memberInfo = new MemberInfo({memberInfoDto: dtos[0], linkTeams: this.self.getSite().canViewDashboard(DashboardConstants.SCOPE_TEAM_AREA), linkProjects: this.self.getSite().canViewDashboard(DashboardConstants.SCOPE_PROJECT_AREA)});
				this.self._widgets.push(memberInfo);
				this.self._infoWidgets[args.userItemId] = memberInfo;
				args.tipSpan.innerHTML = memberInfo.domNode.innerHTML;
				dojo.hitch(args.tooltip,args.tooltip.refresh)();
			},
			failure: function(errorObj) {
				args.tipSpan.innerHTML = dojo.string.substitute(messages.error, [errorObj.message]);
				dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [errorObj]);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		MemberInfoClient.getMemberInfo([args.userItemId], srh, this.getServiceTracker());
	},
	
	_populateRoleTooltip: function(args) {
		var self = this;
		var serviceObj = {
			success: function(/* string */ description) {
				args.tipSpan.className = "com-ibm-team-dashboard-web-viewlets-ui-internal-MembersViewlet-Tooltip";
				if (description)
					args.tipSpan.innerHTML = HTMLUtility.escapeHTML(description);
				else
					args.tipSpan.innerHTML = messages.noDescription;
				args.tooltip.refresh();
			},
			failure: function(error) {
				srh.doErrorCallback(error);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		MemberInfoClient.getRoleDescription(args.roleId, args.processAreaScope, args.processAreaItemId, srh, this.getServiceTracker());
	},
	
	_totalResultsChanged: function() {
		this.updateTotalPageFooter();
		this.getSite().setSecondaryTitle("(" + this._totalResults + ")");
	},
	
	settingsChanged: function(oldSettings, newSettings) {
		var updateTitle = newSettings.userTitle != oldSettings.userTitle;
		if (newSettings.scope != oldSettings.scope || newSettings.scopeItemId != oldSettings.scopeItemId) {
			updateTitle = true;
			this.reset(true);
			this.getSite().refresh();
		}
		else if (newSettings.preferences.items != oldSettings.preferences.items) {
			this.reset(false);
			this.update();
		}
		if (updateTitle)
			this._updateTitle();
	},
	
	_updateTitle: function() {
		this.getSite().setTitle(this.getScopeItem() ? dojo.string.substitute(messages.titleScoped, [this.getScopeItem().name]) : messages.title);
	},
	
	onDestroy: function() {
		this._destroyWidgets();
	},
	
	_destroyWidgets: function() {
		while (this._widgets.length > 0)
			this._widgets.pop().destroy();
	}

});

})();
