/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.TrendReportScrumViewlet");

/*
 * Imports 
 */
dojo.require("dojo.string");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.TrendReportViewlet");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ReportsPreferenceProvider");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility");

(function() {
	
/*
 * Aliases 
 */
var TrendReportViewlet = com.ibm.team.dashboard.viewlets.web.ui.internal.TrendReportViewlet;
var ReportsPreferenceProvider = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.ReportsPreferenceProvider;
var URLUtility = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.URLUtility;

/*
 * TODOs
 */

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.TrendReportScrumViewlet",
	TrendReportViewlet, {
	
	/*
	 * Static Variable Container 
	 */
	_static: function() {}, 
	
	// Viewlet Methods -------------------------------------------------------->
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */		
	init: function() {
		// Call super
		this.inherited(arguments);

		// Static Variables //------------------------------------------------->

		this._initializeStaticConstants();

		// Instance Variables //----------------------------------------------->
		
		// Cascading parameter maps
		this._groupParametersTeam = null;
		this._groupParametersInterval = null;
		
		// Report class
		this._reportClass = this._static._VIEWLET_CLASS_NONE;
		this._setReportClass();
		
		// Preference providers
		this._teamPreferenceProvider = null;
		this._intervalPreferenceProvider = null;

	},	

	/*
 	 * Public Method
 	 * 
 	 * Called to update viewlet content (e.g. Settings changed)
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	refresh: function() {
		// Verify preferences are defined
		try {
			// Team
			this._isStringDefined(this.getPreference(this._static._PREFERENCE_ID_TEAM), true, "team");
			// Interval
			this._isStringDefined(this.getPreference(this._static._PREFERENCE_ID_INTERVAL), true, "interval");
		} catch (errorObject) {
			this._displayMsgSelectPreferences();
			this.getSite().doneRefresh();
			return;
		}

		// Get the rendered report
		this._getReport(
			this.getParameterNames(), 
			this.getParameterValues());		
	},

	/*
 	 * Public Method
 	 * 
 	 * @param String preferenceId
 	 * @return Object
 	 * @overrides super
 	 */		
	getPreferenceProvider: function(preferenceId) {
		
		if (preferenceId == this._static._PREFERENCE_ID_TEAM) {
			// Team
			if (this._teamPreferenceProvider == null) {
				
				var iconMap = new Array();
				iconMap[0] = URLUtility.getContextRoot() + "/web/com.ibm.team.dashboard.web/ui/internal/images/team-dash-icon.gif";
				
				this._teamPreferenceProvider = 
					new ReportsPreferenceProvider({
						preferenceID: preferenceId,
						context: this,
						queryPreferenceMethodName: "_queryPreferenceTeam",
						normalizeMultiValueMethodName: "_normalizePreferenceMultiValue",
						normalizeSingleValueMethodName: "_normalizeTeamAreaPreferenceSingleValue",
						parseValueListMethodName: "_parsePreferenceMultiValue",
						parseValueMapMethodName: "_parsePreferenceMultiValueMap",
						iconMap: iconMap,
						getIconTypeMethodName: "_getIconType",
						isParametersLoadedMethodName: "_isParametersLoadedTeam"
					});
			}
			return this._teamPreferenceProvider;		
				
		} else if (preferenceId == this._static._PREFERENCE_ID_INTERVAL) {
			// Interval
			if (this._intervalPreferenceProvider == null) {
				
				var iconMap = new Array();
				iconMap[0] = URLUtility.getContextRoot() + "/web/com.ibm.team.dashboard.viewlets.web/ui/internal/images/milestone_obj.gif";

				this._intervalPreferenceProvider = 
					new ReportsPreferenceProvider({
						preferenceID: preferenceId,
						context: this,
						queryPreferenceMethodName: "_queryPreferenceInterval",
						normalizeMultiValueMethodName: "_normalizePreferenceMultiValue",
						normalizeSingleValueMethodName: "_normalizeIntervalPreferenceSingleValue",
						parseValueListMethodName: "_parsePreferenceMultiValue",
						parseValueMapMethodName: "_parsePreferenceMultiValueMap",
						iconMap: iconMap,
						getIconTypeMethodName: "_getIconType",
						isParametersLoadedMethodName: "_isParametersLoadedInterval"
					});
			}
			return this._intervalPreferenceProvider;		
		} else {
			var errorObject = new Error(dojo.string.substitute(this.messages.msgUnrecognizedPreference, [preferenceId]));
			this._logError(errorObject);
			this._displayErrorMessage(this.messages.msgPrefLoadingFailed, errorObject);
		}
	},

	// Private Methods -------------------------------------------------------->
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_initializeStaticConstants: function() {
		// Track the number of viewlet instances
		if (this._static._instanceCount == undefined) {
			this._static._instanceCount = 1;
			// Proceed to initialize static constants
		} else {
			this._static._instanceCount++;
			// Static constants already initialized
			return;
		}

		this._static._CLASSNAME = "com.ibm.team.dashboard.viewlets.web.ui.internal.TrendReportScrumViewlet";
		
		// Preferences
		this._static._PREFERENCE_ID_TEAM = "team";	
		this._static._PREFERENCE_ID_INTERVAL = "interval";
		
		// Cascading parameter groups
		this._static._PARAMETER_GROUP_TEAM = "CascadingParameterGroup1";
		this._static._PARAMETER_GROUP_INTERVAL = "CascadingParameterGroup2";
	
		// Viewlet classes
		this._static._VIEWLET_CLASS_NONE = 0;
		this._static._VIEWLET_CLASS_COMPARISON = 1;
		this._static._VIEWLET_CLASS_REST = 2;
		
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Number
 	 * 			 1: Loaded
 	 * 			 0: Not Loaded
 	 * 			-1: Error
 	 */	
	_isParametersLoadedTeam: function() {
		return this._isParametersLoaded(this._groupParametersTeam);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Number
 	 * 			 1: Loaded
 	 * 			 0: Not Loaded
 	 * 			-1: Error
 	 */	
	_isParametersLoadedInterval: function() {
		return this._isParametersLoaded(this._groupParametersInterval);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_setReportClass: function() {
		// Get the current report preference value
		this._reportUUID = this.getPreference(this._staticCommon._PREFERENCE_ID_REPORT);
		
		// Determine report class
		if (this._reportUUID == this._staticCommon._WORK_ITEMS_COMPARISON) {
			// Work Item Comparison
			this._reportClass = this._static._VIEWLET_CLASS_COMPARISON;
			this._setMessageNoResults(this.messages.msgNoOpenWorkItemsFound);
		} else if (this._reportUUID == this._staticCommon._WORK_ITEMS_BURNDOWN) {
			// Sprint Burndown
			this._reportClass = this._static._VIEWLET_CLASS_REST;
			this._setMessageNoResults(this.messages.msgNoWorkItemsEstimatesFound);
		} else if (this._reportUUID == this._staticCommon._WORK_ITEMS_STORY_POINTS) {
			// Story Points
			this._reportClass = this._static._VIEWLET_CLASS_REST;
			this._setMessageNoResults(this.messages.msgNoStoryWorkItemsFound);
		} else if (this._reportUUID == this._staticCommon._WORK_ITEMS_STORY_BY_ITERATION) {
			// Story Points by Iteration
			this._reportClass = this._static._VIEWLET_CLASS_REST;
			this._setMessageNoResults(this.messages.msgNoStoryWorkItemsFound);
		} else if (this._reportUUID == this._staticCommon._WORK_ITEMS_TEAM_VELOCITY) {
			// Team Velocity
			this._reportClass = this._static._VIEWLET_CLASS_REST;
			this._setMessageNoResults(this.messages.msgNoWorkItemsTimeSpentFound);
		} else {
			// BUG: Error message gets overidden in viewlet content area
			var errorObject = new Error(dojo.string.substitute(this.messages.msgUnrecognizedReport, [this._reportUUID]));
			this._logError(errorObject);
			this._displayErrorMessage(this.messages.msgReportSettingFailed, errorObject);
		}
	},		

	/*
 	 * Private Method
 	 * 
	 * @param Object currentSettings
	 * 				. Object preferences
	 *					. <Type-1> <Preference-ID-1>
	 *					. <Type-2> <Preference-ID-2>
	 *					. ...
	 *					. <Type-N> <Preference-ID-N>
	 *				. Number scope
	 *				. String scopeItemId
	 *				. boolean showBackground
	 *				. String title
	 *				. Number trim
	 *				. boolean userTitle
 	 * @return Object
 	 */		
	_queryPreferenceTeam: function(currentSettings) {
		// Team Area
		if (this._groupParametersTeam == null) {
			// The DTO has not been recieved from the REST service call yet.
			// Return null to start an interval-based polling of this method
			// until the DTO is recieved.
			return null;
		} else if (this._groupParametersTeam == this._staticCommon._DTO_PARAMETERS_FAILURE) {
			// Return an error list
			return this._getPreferenceFailureList();
		}
		return this._getPreferenceOptionTeam();
	},
	
	/*
 	 * Private Method
 	 * 
	 * @param Object currentSettings
	 * 				. Object preferences
	 *					. <Type-1> <Preference-ID-1>
	 *					. <Type-2> <Preference-ID-2>
	 *					. ...
	 *					. <Type-N> <Preference-ID-N>
	 *				. Number scope
	 *				. String scopeItemId
	 *				. boolean showBackground
	 *				. String title
	 *				. Number trim
	 *				. boolean userTitle
 	 * @return Object
 	 */		
	_queryPreferenceInterval: function(currentSettings) {
		// Interval
		if (this._groupParametersInterval == null) {
			// The DTO has not been recieved from the REST service call yet.
			// Return null to start an interval-based polling of this method
			// until the DTO is recieved.
			return null;
		} else if (this._groupParametersInterval == this._staticCommon._DTO_PARAMETERS_FAILURE) {
			// Return an error list
			return this._getPreferenceFailureList();
		}
		return this._getPreferenceOptionInterval(currentSettings);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	_resetCascadingParameters: function() {
		this._groupParametersTeam = null;
		this._groupParametersInterval = null;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */	
	_getParameters: function() {
		// Update loading message
		this._updateLoadingMessage(this.messages.msgLoading);
		// Show the loading message
		this._showLoadingArea();
		// Asynchronous call
		this._getGroupParameters(
			this._static._PARAMETER_GROUP_TEAM,
			this._callBackSuccessGetTeamParameters,
			this._callBackErrorGetTeamParameters);
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object oldSettings
 	 * @param Object newSettings
 	 * @return boolean
 	 * @overrides super
 	 */		
	_isSettingsEqual: function(oldSettings, newSettings) {

		if (oldSettings.preferences.team != newSettings.preferences.team) {
			// Team
			return false;
		} else if (oldSettings.preferences.interval != newSettings.preferences.interval) {
			// Interval
			return false;
		} 
		
		return true;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element node
 	 * @return void
 	 * @overrides super
 	 */	
	_addParametersToNode: function(node) {
		// Create parameter names
		var parameterNames = new Array();
		// Create parameter values
		var parameterValues = new Array();		

		// Team
		var team = this.getPreference(this._static._PREFERENCE_ID_TEAM);
		team = this._normalizeMultiValue(team, "_normalizeContentAreaSingleTeam");
		parameterNames.push(this.messages.msgTeamArea);
		parameterValues.push(team);
		
		// Interval
		var interval = this.getPreference(this._static._PREFERENCE_ID_INTERVAL);		
		interval = this._normalizePreferenceMultiValue(interval);
		parameterNames.push(this.messages.msgIteration);		
		parameterValues.push(interval);

		this._displayParameters(node, parameterNames, parameterValues);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Array : String
 	 */	
	getParameterNames: function() {
		
		if (this._reportClass == this._static._VIEWLET_CLASS_COMPARISON) {
			return ["ProjectAreaName", "TeamArea", "ProjectAreaName1", "Interval", this._staticCommon._PARAMETER_NAME_ZOOM];
		}
		return ["ProjectAreaName", "TeamAreaName", "ProjectAreaName1", "Interval", this._staticCommon._PARAMETER_NAME_ZOOM];
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Array
 	 */	
	getParameterValues: function() {
		// Example
		// https://jazz.net/jazz/secure/service/com.ibm.team.reports.common.internal.service.IReportRestService/renderedReport?reportUUID=_UBEVoOpcEdygpId1BsBPUw
		// embeddable=true
		// parameterName=ProjectAreaName
		// parameterName=TeamAreaName
		// parameterName=ProjectAreaName1
		// parameterName=Interval
		// parameterValue='%7BCurrent%20Project%20Area%7D'
		// parameterValue='%2FJazz%20Development%2FDashboard'
		// parameterValue='%7BCurrent%20Project%20Area%7D'
		// parameterValue='0.6%20M5'%2C'%7BCurrent%20Iteration%7D'
		// webui=true
		// showArchived=false
		
		// Example
		// Multiple parameter value:  '/Jazz/Build','/Jazz/Dashboard'
		// &parameterValue='%2FJazz%2FBuild'%2C'%2FJazz%2FDashboard'
		
		// Note:  No need to encode the parameter values; since, this will be
		// done when making the REST service call
		
		var parameterValues = new Array();
		
		// Project
		parameterValues.push(this._quote(this._staticCommon._SPECIAL_PROJECT_AREA));
		
		// Team
		var team = this.getPreference(this._static._PREFERENCE_ID_TEAM);
		team = this._normalizeParameterMultiValue(team);
		parameterValues.push(team);

		// Project 2
		parameterValues.push(this._quote(this._staticCommon._SPECIAL_PROJECT_AREA));
		
		// Interval
		var interval = this.getPreference(this._static._PREFERENCE_ID_INTERVAL);		
		interval = this._normalizeParameterMultiValue(interval);
		parameterValues.push(interval);

		// Zoom
		parameterValues.push(this._quote(this.isZoomed()));
	
		return parameterValues;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String groupName 
 	 * @param com.ibm.team.reports.rest:ReportParameterGroupDTO dto
 	 * @return void
 	 */
	_callBackSuccessGetTeamParameters: function(self, groupName, dto) {
		// Create a map out of the DTO
		var map = self._processDTOGroupParameters(dto);
		// Store global
		self._groupParametersTeam = map;
		// Debug
		// console.debug(dto); 
		// self._debugCascadingParameters(map, "", 0);
		
		// Chain parameter call
		// Asynchronous call
		self._getGroupParameters(
			self._static._PARAMETER_GROUP_INTERVAL,
			self._callBackSuccessGetIntervalParameters,
			self._callBackErrorGetIntervalParameters);
		
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String parameterName 
 	 * @param com.ibm.team.reports.rest:ReportParameterDTO dto
 	 * @return void
 	 */
	_callBackSuccessGetIntervalParameters: function(self, parameterName, dto) {
		// Create a map out of the DTO
		var map = self._processDTOGroupParameters(dto);
		// Store global
		self._groupParametersInterval = map;
		// Debug
		// console.debug(dto);
		// self._debugCascadingParameters(map, "", 0);
		
		// Show the display area
		self._showDisplayArea();
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String parameterName 
 	 * @return void
 	 */
	_callBackErrorGetIntervalParameters: function(self, parameterName) {
		// Set error
		self._groupParametersInterval = self._staticCommon._DTO_PARAMETERS_FAILURE;
	},			

	/*
 	 * Private Method
 	 * 
 	 * @param Object self 
 	 * @param String groupName 
 	 * @return void
 	 */
	_callBackErrorGetTeamParameters: function(self, groupName) {
		// Set error
		self._groupParametersTeam = self._staticCommon._DTO_PARAMETERS_FAILURE;
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Object
 	 */		
	_getPreferenceOptionTeam: function() {
		// Get parameters
		var projectAreas = this._groupParametersTeam;
		var teamArea = null;
		try {
			// Get team area
			teamArea = projectAreas[this._staticCommon._SPECIAL_PROJECT_AREA];
		} catch (errorObject) {
			return this._getPreferenceFailureList();
		}
		// Adapt parameters to preferences
		var result = this._adaptParametersToPreferences(teamArea, "_normalizeTeamAreaPreferenceSingleValue");
		
		return result;
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return Object
 	 */		
	_getPreferenceOptionInterval: function() {
		// Get parameters
		var projectAreas = this._groupParametersInterval;
		var interval = null;
		try {
			// Get team area
			interval = projectAreas[this._staticCommon._SPECIAL_PROJECT_AREA];
		} catch (errorObject) {
			return this._getPreferenceFailureList();
		}
		// Adapt parameters to preferences
		var result = this._adaptParametersToPreferences(interval, "_normalizeIntervalPreferenceSingleValue");
		
		return result;
	},	
	
	// Class Methods ---------------------------------------------------------->
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return String
 	 */		
	toString: function() {
		return this._static._CLASSNAME;
	}

});


})();

