/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.ViewletChooserBlock");
/*
 * Imports 
 */
dojo.require("dojo.i18n"); 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.Tree");
dojo.require("dojo.data.ItemFileReadStore");
dojo.require("dojo.parser");

dojo.require("com.ibm.team.dashboard.web.client.internal.ViewletRegistry");
dojo.require("com.ibm.team.dashboard.web.ui.internal.ViewletTree");
dojo.require("com.ibm.team.dashboard.web.util.internal.HTMLUtility");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

/*
 * Aliases 
 */
var HTMLUtility = com.ibm.team.dashboard.web.util.internal.HTMLUtility;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ViewletRegistry = com.ibm.team.dashboard.web.client.internal.ViewletRegistry;
var ViewletTree = com.ibm.team.dashboard.web.ui.internal.ViewletTree;

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.ViewletChooserBlock",
	[dijit._Widget, dijit._Templated], {

	/*
 	 * Template
 	 */
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/ViewletChooserBlock.html"),
	
	/*
 	 * Constructor
 	 * 
 	 * @param Object args
 	 * 			Property self
 	 * @return <none> 
 	 */
	constructor: function(args) {
		
		// Constants //------------------------------------------------------------>
		
		this._CLASSNAME = "com.ibm.team.dashboard.web.ui.internal.ViewletChooserBlock";
		
		this._PRE_VIEWLET_ICON_SEGMENT = "../../web/";
		
		this._INTRO_DETAILS_PANE_KEY = "intro";
		this._CATEGORY_DETAILS_PANE_KEY = "category";
		this._ITEM_DETAILS_PANE_KEY = "viewlet";
		
		this._ITEM_TREE_NODE_ID = "viewlet";
		this._CATEGORY_TREE_NODE_ID = "category";
		
		// Instance Variables //--------------------------------------------------->
		
		// Container to add viewlets to
		this._container = args.self;
		// Target dashboard scope
		this._scope = args.scope;
		// Target dashboard scope item id
		this._scopeItemId = args.scopeItemId;
		// Viewlet hierarchy DTO root
		this._dto = null;
		// Viewlet tree widget
		this._viewletTree = null;
		// Container for detail panes
		this._detailPanes = new Array();
		// Key of current details pane selected
		this._currentDetailPaneKey = null;		
		// Intro Details Pane
		this._introDetailPane = null;
		// Category Details Pane
		this._categoryDetailPane = null;
		// Item Details Pane
		this._itemDetailPane = null;
		
		this._nameMap = new Array();

		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		this.loading = this.messages.loading;
		this.selectFromChooser = this.messages.selectFromChooser;
		this.viewletCategoryDescription = this.messages.viewletCategoryDescription;
		this.appliesTo = this.messages.appliesTo;
		this.description = this.messages.description;
		this.preview = this.messages.preview;
		this.icon = this.messages.icon;
		this.viewletPreviewImage = this.messages.viewletPreviewImage;
		this.addViewletText = this.messages.addViewletText;
		this.msgClose = this.messages.close;
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 * @overrides super
 	 */		
	postCreate: function() {
		this._initialize();
	},	
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * @return void
 	 */		
	startup: function() {
		if (!this._started) {
			this._started = true;
			this._getViewletHierarchy();
		}
	},
	
	/*
 	 * Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */
	_initialize: function() {
		// Register detail panes
		this._registerDetailPanes();
		// Get the viewlet hierarchy and create the tree
		this._showLoadingMessage();
		// Initialize sites
		this._initCloseButtonSite();
		// Initialize panes
		this._initIntroDetailPane();
		this._initCategoryDetailPane();
		this._initItemDetailPane();
	},
	
	/*
 	 * Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */
	_registerDetailPanes: function() {
		// Intro Details Pane
		this._detailPanes[this._INTRO_DETAILS_PANE_KEY] = this._viewletIntroPane;
		// Category Details Pane
		this._detailPanes[this._CATEGORY_DETAILS_PANE_KEY] = this._viewletCategoryPane;
		// Item Details Pane
		this._detailPanes[this._ITEM_DETAILS_PANE_KEY] = this._viewletItemPane;			
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */
	_initCloseButtonSite: function() {
		var self = this;
		
		this.connect(this._closeButton, "onclick", function() {
			// Close chooser
			self._container.closeViewletChooser();
		});		
		this.connect(this._closeButton, "onmouseover", function() {
			// Highlight on
			self._closeButton.src = "../../web/com.ibm.team.dashboard.web/ui/internal/images/close-hov.gif";
		});
		this.connect(this._closeButton, "onmouseout", function() {
			// Highlight off
			self._closeButton.src = "../../web/com.ibm.team.dashboard.web/ui/internal/images/close.gif";
		});		
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */
	_initIntroDetailPane: function() {
		// Set the initial pane to be the intro
		this._currentDetailPaneKey = this._INTRO_DETAILS_PANE_KEY;
		// Get the intro detail pane
		var pane = this._getDetailPane(this._currentDetailPaneKey);
		
		// Title
		this._headerTitle.innerHTML = this.messages.viewletChooser;
		// Icon
		this._headerIcon.src = "../../web/com.ibm.team.dashboard.web/ui/internal/images/tree-category-icon.gif";
		// Button
		this._updateVisibility(this._viewletButton, false);
		
		// Show the intro pane
		this._updateVisibility(pane, true);
	},	
	
	/*
 	 * Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */
	_initCategoryDetailPane: function() {
		// NO-OP
	},		
	
	/*
 	 * Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */
	_initItemDetailPane: function() {
		// Create the Add button
		this._createButtonAddViewlet();		
	},	
	
	/*
 	 * Method
 	 * 
 	 * @param String key
 	 * @return Element
 	 */
	_getDetailPane: function(key) {
		return this._detailPanes[key];
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return <none>
 	 */
	_selectionChanged: function(event) {
		// Update the current detail pane shown
		this._updateDetailsSection(event);
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return <none>
 	 */
	_doubleClickOccurred: function(event) {
		// Note: _selectionChanged fired on first click
		// No need to process event
		this._handleEventButtonClicked();		
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Element node
 	 * @param boolean show
 	 * @return <none>
 	 */	
	_updateVisibility: function(node, show) {
		if (show == true) {
			// Show
			node.style.display = "block";
		} else {
			// Hide
			node.style.display = "none";
		}
	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_showLoadingMessage: function() {
		this._updateVisibility(this._treeMessage, true);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_hideLoadingMessage: function() {
		this._updateVisibility(this._treeMessage, false);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * 
 	 * @return boolean
 	 */	
	_updateDetailsSection: function(event) {
		// Get the node type selected
		var type = event.type;
		
		// TODO: MP: Call with interface method "update" on separate classes
		// Update the detail pane contents
		if (type == this._CATEGORY_TREE_NODE_ID) {	
			// Category
			this._updateCategoryDetailPane(event);
		} else if (type == this._ITEM_TREE_NODE_ID) {
			// Item
			this._updateItemDetailPane(event);
		}

		// Get the current detail pane selected
		var oldPane = this._getDetailPane(this._currentDetailPaneKey);
		// Hide the old pane
		this._updateVisibility(oldPane, false);
		// Get the new details pane selected
		var newPane = this._getDetailPane(type);
		// Show the new pane
		this._updateVisibility(newPane, true);		
		// Set the new pane as the current detail pane
		this._currentDetailPaneKey = type;

	},		
	
	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * 
 	 * @return void
 	 */
	_updateCategoryDetailPane: function(event) {
		// Bread Crumbs
		this._categoryTitle.innerHTML = "";
		this._categoryTitle.style.marginTop = "0";
		this._categoryTitle.style.marginBottom = "0";
		// Title
		this._headerTitle.innerHTML = HTMLUtility.escapeHTML(event.name);
		// Icon
		this._headerIcon.src = "../../web/com.ibm.team.dashboard.web/ui/internal/images/folder.gif";
		// Button
		this._updateVisibility(this._viewletButton, false);
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * 
 	 * @return void
 	 */
	_updateItemDetailPane: function(event) {
		this._updateDetailsSectionFields(event);
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * 
 	 * @return void
 	 */	
	_updateDetailsSectionFields: function(event) {
		// Update the icon field
		this._updateDetailsIconField(event);
		// Update the name field
		this._updateDetailsNameField(event);
		// Button
		this._updateVisibility(this._viewletButton, true);
		// Update the category field
		this._updateDetailsCategoryField(event);
		// Update the applicability field
		this._updateDetailsAppliesToField(event);
		// Update the description field
		this._updateDetailsDescriptionField(event);
		// Update the preview field
		this._updateDetailsPreviewField(event);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * 
 	 * @return void
 	 */	
	_updateDetailsIconField: function(event) {
		// Get the icon path
		var iconPath = event.icon[0];
		// Assemble the src
		var src;
		if (iconPath.indexOf("://") == -1) {
			// e.g com.ibm.team.dashboard.viewlets.web/ui/internal/images/icon-wiki.gif
			src = this._PRE_VIEWLET_ICON_SEGMENT + iconPath;
		} else {
			// e.g. https://jazz.net/favicon.ico
			src = iconPath;
		}
		// Set the src
		this._headerIcon.src = src;
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * 
 	 * @return void
 	 */	
	_updateDetailsNameField: function(event) {
		// Get the name of the item selected
		var name = event.name;
		// Set the name
		this._headerTitle.innerHTML = HTMLUtility.escapeHTML(name);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * 
 	 * @return void
 	 */	
	_updateDetailsCategoryField: function(event) {
		// Bread Crumbs
		this._categoryTitle.style.marginTop = "7px";
		this._categoryTitle.style.marginBottom = "7px";
		// Set the category (already HTML escaped)
		this._categoryTitle.innerHTML = event.category;
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return void
 	 */	
	_updateDetailsAppliesToField: function(event) {
		// Get the applicability of the item selected
		var rawAppliesTo = event.applicability;		
		// Normalize the value
		var normalizedAppliesTo = null;
		if (rawAppliesTo == "teams") {
			normalizedAppliesTo = this.messages.appliesToTeams;
		} else if (rawAppliesTo == "projects") {
			normalizedAppliesTo = this.messages.appliesToProjects;
		} else if (rawAppliesTo == "projects-and-teams") {
			normalizedAppliesTo = this.messages.appliesToProjectsTeams;
		} else if (rawAppliesTo == "contributors") {
			normalizedAppliesTo = this.messages.appliesToContributors;
		} else if (rawAppliesTo == "all") {
			normalizedAppliesTo = this.messages.appliesToAll;
		} else {
			normalizedAppliesTo = this.messages.appliesToUnknown;
		}
		// Get the applicability field node		
		var applicabilityFieldNode = "_detailsFieldAppliesTo";
		var node = dojo.byId(applicabilityFieldNode);
		// Set the applicability
		node.innerHTML = normalizedAppliesTo;
	},	
	
	/*
 	 * Method
 	 * 
 	 * @param event - Event
 	 * @return <none>
 	 */	
	_updateDetailsDescriptionField: function(event) {
		// Get the description of the item selected
		var description = event.description;		
		// Ensure a description was provided
		if (description == "undef") {
			description = this.messages.noDescriptionSpecified;
		}
		// Get the description field node		
		var descriptionFieldNode = "_detailsFieldDescription";
		var node = dojo.byId(descriptionFieldNode);
		// Set the description
		node.innerHTML = HTMLUtility.sanitizeHTML(description);
	},

	/*
 	 * Method
 	 * 
 	 * @param event - Event
 	 * @return <none>
 	 */	
	_updateDetailsPreviewField: function(event) {
		// Get the preview image of the item selected
		var preview = event.preview;
		// Resolve the preview image path
		var previewImagePath = this._resolvePreviewImagePath(preview);
		// Get the preview field node		
		var previewFieldNode = "_detailsFieldPreview";
		var node = dojo.byId(previewFieldNode);
		// Set the preview image path		
		node.setAttribute("src", previewImagePath);
	},	

	/*
 	 * Method
 	 * 
 	 * @param String: preview
 	 * @return String
 	 */	
	_resolvePreviewImagePath: function(preview) {
		// The default preview image
		var previewImagePath = "../../web/com.ibm.team.dashboard.viewlets.web/ui/internal/images/preview-default.gif";
		// Check to see if a preview image was specified			
		if ((preview == null) ||
			(preview == false)) {
			return previewImagePath;
		}
		// Preview Format: com.ibm.team.dashboard.viewlets.web/ui/internal/images/icon-wiki.gif
		previewImagePath = "../../web/" + preview;
		
		return previewImagePath;				 
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */          
	_createButtonAddViewlet: function() {
		this.connect(this._viewletButton, "onclick", this._handleEventButtonClicked);
	},	

	/*
 	 * Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */	
	_handleEventButtonClicked: function() {
		// Note:  Viewlet type has to be 'viewlet' if button clicked
		// Get the selected tree node
		var selectedNode = this._getSelectedTreeNode();
		// Ensure we have a node
		if (selectedNode == null) {
			return;
		}		
		// Get the node name
		var name = selectedNode.name;
		// Ensure we have a value
		if (name == null) {
			return;
		}
		if (typeof(name) != "string") {
			name = "" + name;
		}
		// Get the node id
		var id = selectedNode.id;
		// Ensure we have a value
		if (id == null) {
			return;
		}
		if (typeof(id) != "string") {
			id = "" + id;
		}
		var icon = selectedNode.icon + "";
		var preferences = new Object();
		for (var i=0;i<selectedNode.preferences.length;i++) {
			var pref = selectedNode.preferences[i];
			var index = pref.indexOf("=");
			preferences[pref.substring(0, index)] = pref.substring(index + 1);
		}
		var showBackground = selectedNode.showBackground[0];
		var trim = selectedNode.trim[0];
		
		// Add the selected viewlet to the dashboard
		this._container.addViewlet(name, id, icon, preferences, null, null, null, showBackground, trim, null, 0, 0, false);
	},
	
	/*
 	 * Method
 	 * 
 	 * @param <none>
 	 * @return Object (Fields:  id, name, type) 
 	 */	
	_getSelectedTreeNode: function() {
		var tree = this._viewletTree;
		// Ensure the tree is defined
		if (tree == null) {
			return null;
		}
		// Get the last focus node
		var lastFocusNode = tree.lastFocused;
		// Ensure the node is defined
		if (lastFocusNode == null) {
			return null;
		}
		// Get the item
		return lastFocusNode.item;
	},
	
	/*
 	 * Method
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */
	_getViewletHierarchy: function() {
		// Create the service object
		var serviceObject = {
			_chooser: this,
			// Service success
			success: function(/* ViewletHierarchyDTO */ root) {
				// Note:  Because of multi-threaded race condition this variable
				// is not available to the client when creating the viewlet
				// hierarchy tree
				this._chooser._dto = root;
				// Debug the viewlet hierarchy
				//this._chooser._debugViewletHierarchyDTO(root);
				// Create the viewlet hierarchy tree
				this._chooser._createViewletHierarchyTree(root);	

				this._chooser._hideLoadingMessage();
			},
			// Service failure
			failure: function(errorObject) {
				this._chooser._hideLoadingMessage();
				this._chooser._logError(errorObject);
			}
		};
		// Create the handler
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		// Get the chooser info
		ViewletRegistry.getHierarchy(this._scope, this._scopeItemId, handler);	
	},

	/*
 	 * Method
 	 * 
 	 * @param ViewletHierarchyDTO
 	 * @return <none>
 	 */
	_createViewletHierarchyTree: function(root) {
		// Validate root
		if (root == null) {
			return;
		}
		// Get the attach point
		var node = this._viewletChooser; 
		// Create the data store using the DTO root
		var dataStore = this._createViewletHierarchy(root);
		// Create the tree store using the data store
        var treeStore = new dojo.data.ItemFileReadStore({data: dataStore});
  		// Create the tree
		this._viewletTree = new ViewletTree({
				store: treeStore,
				widgetId: 'viewletTree',
				persist: false
			}
		);
		node.appendChild(this._viewletTree.domNode);
		// Register the chooser to recieve tree selection events
		this._viewletTree._addSelectionListener({self: this});
		// Register the chooser to recieve tree double click events
		this._viewletTree._addDoubleClickListener({self: this});
	},	

	/*
 	 * Method
 	 * 
 	 * @param ViewletHierarchyDTO
 	 * @return Object
 	 */
	_createViewletHierarchy: function(root) {
		// Create the array for tree items
		var items = new Array();
		// Get the root categories
		var categories = root.viewletCategories;
		// Get the category count
		var categoryCount = 0;
		if (categories != null) {
			categoryCount = categories.length;
		}
		// Process categories
		for (var i = 0; i < categoryCount; i++) {
			// Get the current category
			var category = categories[i];
			// Create the tree item out of the category DTO
			items[i] = this._createViewletCategory(category, null);
		}
		// Sort child items
		items.sort(this._sortViewletEntries);		
		// Create the data store object
		var dataStore = new Object();
		// Set properties
		dataStore.label = 'name';
        dataStore.identifier = 'name';
		
		if (items.length > 0) {
			dataStore.items = items;
		}
		
		return dataStore;
	},

	/*
 	 * Method
 	 * 
 	 * @param ViewletCategoryDTO: category
 	 * @param String: categoryCrumbs
 	 * @return Object
 	 */
	_createViewletCategory: function(category, categoryCrumbs) {
		// Create the array for tree items
		var children = new Array();
		// Get the child categories
		var categories = category.viewletCategories;
		// Get the category count
		var categoryCount = 0;
		if (categories != null) {
			categoryCount = categories.length;
		}
		// Update the category bread crumbs
		if ((categoryCrumbs == null) ||
			(categoryCrumbs.length == 0)) {
			categoryCrumbs = HTMLUtility.escapeHTML(category.name);
		} else {
			categoryCrumbs = categoryCrumbs + " &gt; " + HTMLUtility.escapeHTML(category.name);
		}
		// Process child categories
		for (var i = 0; i < categoryCount; i++) {
			// Get the current category
			var childCategory = categories[i];
			// Create the tree item out of the category DTO
			children[i] = this._createViewletCategory(childCategory, categoryCrumbs);
		}
		// Sort child items
		children.sort(this._sortViewletEntries);				
		// Get the child items
		var items = category.viewletItems;
		// Get the category count
		var itemCount = 0;
		if (items != null) {
			itemCount = items.length;
		}	
		// Process child items		
		var viewletChildren = new Array();
		for (var i = 0; i < itemCount; i++) {
			// Get the current item
			var childItem = items[i];
			// Create the tree item out of the item DTO
			viewletChildren.push(this._createViewletItem(childItem, categoryCrumbs));
		}	
		// Sort child items
		viewletChildren.sort(this._sortViewletEntries);
		// Merge viewlets with categories
		this._mergeArrays(children, viewletChildren);
		
		// Create the data store object
		var dataStoreCategory = new Object();
		// Set properties
		dataStoreCategory.name = this._getNodeName(category.name);
		dataStoreCategory.type = this._CATEGORY_TREE_NODE_ID;
		
		if (children.length > 0) {
			dataStoreCategory.children = children;
		}
		
		return dataStoreCategory;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object entry1
 	 * @param Object entry2
 	 * @return <none>
 	 */
	_sortViewletEntries: function(entry1, entry2) {
		if (entry1.name < entry2.name) {
			// Less than
			return -1;
		} else if (entry1.name > entry2.name) {
			// Greater than
			return 1;
		}
		// Equal
		return 0;
	},

	/*
 	 * Private Method
 	 * 
 	 * Merges array2 into array1
 	 * 
 	 * @param Array array1
 	 * @param Array array2
 	 * 
 	 * @return void
 	 */
	_mergeArrays: function(array1, array2) {
		for (var index = 0; index < array2.length; index++) {
			array1.push(array2[index]);
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String name
 	 * @return String
 	 */
	_getNodeName: function(name) {
		// Workaround to Dojo data store "identifier" and "label" bug
		// Make the names unique
		var nodeName = name;
		if (nodeName in this._nameMap) {
			for (var i = 0; i < this._nameMap[nodeName]; i++) {
				nodeName = nodeName + " ";
			}
			this._nameMap[nodeName]++;
		} else {
			this._nameMap[nodeName] = 1;
		}
		return nodeName;		
	},

	/*
 	 * Method
 	 * 
 	 * @param ViewletItemDTO: item
 	 * @param String: categoryCrumbs
 	 * @return Object
 	 */
	_createViewletItem: function(item, categoryCrumbs) {
		// Create the data store object
		var dataStoreItem = new Object();
		// Set properties
		dataStoreItem.name = this._getNodeName(item.name);
		dataStoreItem.type = this._ITEM_TREE_NODE_ID;
		dataStoreItem.id = item.viewletDefId;
		dataStoreItem.category = categoryCrumbs;
		dataStoreItem.description = item.description;
		dataStoreItem.icon = item.icon;
		dataStoreItem.preview = item.preview;
		dataStoreItem.applicability = item.appliesTo;
		dataStoreItem.showBackground = item.showBackground;
		dataStoreItem.trim = item.trim;
		
		/*
		 * Save preferences in data store as an array of "id=value"
		 * strings (saving as object with properties does not seem to
		 * work(?)).
		 */
		dataStoreItem.preferences = new Array();
		if (item.preferences) {
			for (var i=0;i<item.preferences.length;i++) {
				var pref = item.preferences[i];
				dataStoreItem.preferences.push(pref.id + "=" + pref.value);
			}
		}
		
		return dataStoreItem;		
	},

	/*
 	 * Method
 	 * 
 	 * @param ViewletHierarchyDTO
 	 * @return <none>
 	 */
	_debugViewletHierarchyDTO: function(root) {
		if (root == null) {
			return;
		}
		var categories = root.viewletCategories;
		var text = document.createTextNode(this._debugDTOCategories(categories));
		var node = this._viewletChooser;  
		node.appendChild(text);
	},	
	
	/*
 	 * Method
 	 * 
 	 * @param ViewletCategoryDTO[]
 	 * @return String
 	 */
	_debugDTOCategories: function(categories) {
		var output = "";
		for (var x in categories) {
			var category = categories[x];
			output += "Category = " + category.name;
			output += " [ ";			
			var cats = category.viewletCategories;
			if (cats) {
				output += this._debugDTOCategories(cats);
			}
			var items = category.viewletItems;
			output += " ] ";
			output += " [ ";			
			if (items) {
				output += this._debugDTOItems(items);
			}
			output += " ] ";
		}
		return output;
	},
	
	/*
 	 * Method
 	 * 
 	 * @param ViewletItemDTO[]
 	 * @return String
 	 */
	_debugDTOItems: function(items) {
		var output = "";
		for (var x in items) {
			var item = items[x];
			output += "Item = " + item.name + " ";
		}
		return output;
	},
	
	/*
 	 * Method
 	 * 
 	 * @param Error
 	 * @return <none>
 	 */
	_logError: function(errorObject) {
		var errorMessage = "REMOTE ERROR: " + this._CLASSNAME + ": " + errorObject.message;
		console.error(errorMessage);
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param <none>
 	 * @return String
 	 */		
	toString: function() {
		return this._CLASSNAME;
	},
	
	destroy: function() {
		if (this._viewletTree && this._viewletTree.destroy)
			this._viewletTree.destroy();
		this.inherited(arguments);
	}
});


})();
