/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.WorkItemsViewlet");

dojo.require("dojo.i18n");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.PagedTableViewlet");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.WorkItemQueryPreferenceProvider");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.FormatUtil");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemComment");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.process.web.client.projectarea");

dojo.require("com.ibm.team.workitem.web.client.internal.Constants");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemAttributes");

dojo.require("dojo.date");
dojo.require("dojo.date.locale");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "WorkItemsMessages");

(function() {
	
var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;

var PagedTableViewlet = com.ibm.team.dashboard.viewlets.web.ui.internal.PagedTableViewlet;
var WorkItemQueryPreferenceProvider = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.WorkItemQueryPreferenceProvider;

var Constants = com.ibm.team.workitem.web.client.internal.Constants;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var QueryClient = com.ibm.team.workitem.web.client.internal.QueryClient;
var WorkItemClient = com.ibm.team.workitem.web.client.internal.WorkItemClient;
var WorkItemUtil = com.ibm.team.workitem.web.client.util;
var FormatUtil = com.ibm.team.dashboard.web.ui.internal.util.FormatUtil;

var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

var WorkItemAttributes = com.ibm.team.workitem.web.client.internal.WorkItemAttributes;

var IMG_PATH = "../../web/com.ibm.team.dashboard.viewlets.web/ui/internal/images/";
var IMG_URL_CLOSED_OVERLAY = IMG_PATH + "wi_complete_ovr.gif";
var IMG_URL_IN_PROGRESS_OVERLAY = IMG_PATH + "wi_in_progress_ovr.gif";

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.WorkItemsViewlet",
	PagedTableViewlet, {
	
	onInit: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "WorkItemsMessages");
		// if the viewlet has a default query and it has not been changed, initialize the title and preference provider
		var defaultQuery = this.getPreference("defaultQuery");
		if (defaultQuery != -1 && this.getPreference("query") == -1) {
			var serviceObject = {
				self: this,
				_successCallback: function(/*array*/ queryDTO) {
					if (!this.self.isDestroyed() && queryDTO) {
						this.self._defaultQueryItemId = queryDTO.itemId;
						this.self.getSite().setTitle(queryDTO.name);
						this.self.getPreferenceProvider("query").initDefault(queryDTO);
					}
				},
				_errorCallback: function(error) {
					dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [error]);
				}
			};
			var responseHandler = new ServiceResponseHandler(serviceObject, "_successCallback", "_errorCallback");
			var args = {
				includeCount: false,
				id: defaultQuery
			};
			QueryClient.getQuery2(responseHandler, args);
		}
	},
	
	getResultCount: function() {
		if (this.getPreference("query") == -1 && this.getPreference("defaultQuery") == -1)
			return 0;
		return (this._totalResults || this._totalResults == 0) ? this._totalResults : -1;
	},
	
	getNoResultsMessage: function() {
		if (this._deletedQuery)
			return this._getDeletedQueryMessage();
		if (this.getPreference("query") != -1 || this.getPreference("defaultQuery") != -1)
			return document.createTextNode(this.messages.noResults);
		return this._getNoQueryMessage();
	},
	
	setDashboardEditing: function(editing) {
		if (this._noQueryEditMessage)
			this._noQueryEditMessage.className = editing ? "" : "hidden";
		if (this._deletedQueryEditMessage)
			this._deletedQueryEditMessage.className = editing ? "" : "hidden";
	},
	
	_getDeletedQueryMessage: function() {
		if (!this._deletedQueryMessage) {
			this._deletedQueryMessage = document.createElement("span");
			this._deletedQueryMessage.appendChild(document.createTextNode(this.messages.deletedQuery));
			this._deletedQueryEditMessage = this._createEditSettingsMessage(true);
			this._deletedQueryMessage.appendChild(this._deletedQueryEditMessage);
		}
		return this._deletedQueryMessage;
	},
	
	_getNoQueryMessage: function() {
		if (!this._noQueryMessage) {
			this._noQueryMessage = document.createElement("span");
			this._noQueryMessage.appendChild(document.createTextNode(this.messages.noQuery));
			this._noQueryEditMessage = this._createEditSettingsMessage(false);
			this._noQueryMessage.appendChild(this._noQueryEditMessage);
		}
		return this._noQueryMessage;
	},
	
	_createEditSettingsMessage: function(/* boolean */ isDeleted) {
		var result = document.createElement("span");
		var links = I18nUtil.addMessage(result, isDeleted ? this.messages.selectNewQuery : this.messages.selectQuery);
		this.connect(links[0], "onclick", this._doPreferences);
		return result;
	},
	
	_doPreferences: function(event) {
		dojo.stopEvent(event);
		
		this.getSite().toggleSettings();
	},
	
	getItemsPerPage: function() {
		return this.getPreference("items");
	},
	
	getContentClass: function() {
		return "WorkItemsViewlet";
	},
	
	onRefresh: function() {
		if (this._detailsWidgets) {
			for (var i in this._detailsWidgets)
				this._detailsWidgets[i].destroy();
		}
		this._detailsWidgets = new Object();
		delete this._totalResults;
		delete this._token;
		delete this._deletedQuery;
	},
	
	_totalResultsChanged: function() {
		this.updateTotalPageFooter();
		this.getSite().setSecondaryTitle("(" + this._totalResults + ")");
	},
	
	fetch: function(srh, start, count, entries, details) {
		var serviceObject = {
			self: this,
			success: function(resultSet){
				if (!this.self.isDestroyed()) {
					if (resultSet && resultSet.totalCount > 0) {
						this.self._resultHeaderIndex = {};
						this.self._resultHeaders = resultSet.headers;
						for (var i = 0; i < resultSet.headers.length; i++) 
							this.self._resultHeaderIndex[resultSet.headers[i].attributeId] = i;
						if (this.self._totalResults != resultSet.totalCount) {
							this.self._totalResults = resultSet.totalCount;
							this.self._totalResultsChanged();
						}
						this.self._token = resultSet.token;
						if (resultSet.rows) {
							var dtos = resultSet.rows;
							var result = new Object();
							if (entries) 
								result.entries = dtos;
							if (details) 
								result.details = dtos;
							srh.doCallback(result);
						}
					}
					else {
						this.self._totalResults = 0;
						this.self._totalResultsChanged();
						srh.doCallback();
					}
				}
			},
			failure: function(error){
				// 404 means query from settings area does not exist
				// 400 means defaultQuery from template does not exist
				if (error.status == 404 || error.status == 400) {
					this.self._deletedQuery = true;
					this.self._totalResults = 0;
					this.self.updateTotalPageFooter();
					this.self.getSite().setSecondaryTitle("");
					srh.doCallback();
				}
				else {
					srh.doErrorCallback(error);
				}
			}
		};
		var responseHandler = new ServiceResponseHandler(serviceObject, "success", "failure");
		var items = this.getPreference("items");
		var args = {
			startIndex: start,
			maxResults: count,
			projectAreaItemId: getCurrentProjectArea().itemId,
			columnIdentifiers: [WorkItemAttributes.TYPE, WorkItemAttributes.SUMMARY, WorkItemAttributes.STATE,
				WorkItemAttributes.CREATOR, WorkItemAttributes.OWNER, WorkItemAttributes.SEVERITY, WorkItemAttributes.PRIORITY,
				WorkItemAttributes.CREATION_DATE, WorkItemAttributes.RESOLUTION, WorkItemAttributes.ESTIMATE,
				WorkItemAttributes.CATEGORY, WorkItemAttributes.TARGET, WorkItemAttributes.PROJECT_AREA, WorkItemAttributes.TAGS]
		};
		var queryItemId = this.getPreference("query");
		if (queryItemId == -1) {
			var queryStringId = this.getPreference("defaultQuery");
			if (queryStringId == -1) {
				srh.doCallback();
				return;
			}
			args.id = queryStringId;
		} else {
			args.itemId = queryItemId;
		}
		if (this._token)
			args.token = this._token;
		QueryClient.getResultSet(null, args, responseHandler);
	},
	
	/* Node[] */ renderEntry: function(/* object */ resultDTO, /* object */ resultDetailsDTO, /* boolean */ zoomed) {
		var columns = new Array();
		
		var titleColSpan = 2;
		
		var href = WorkItemUtil.getViewWorkItemURI(resultDTO.id);
		
		var iconCell = document.createElement("td");
		var iconContainer = document.createElement("div");
		iconContainer.className = "iconContainer";
		
		iconCell.className = "iconCell";
		var icon = document.createElement("img");
		icon.width = 16;
		icon.height = 16;
		icon.style.marginRight = "0.2em";
		icon.src = resultDTO.iconUrls[this._resultHeaderIndex[WorkItemAttributes.TYPE]];
		iconContainer.appendChild(icon);
		
		// add overlay if available/needed
		if (resultDTO.stateGroup) {
			var src;
			switch (resultDTO.stateGroup) {
				case Constants.CLOSED_STATES:	
					src = IMG_URL_CLOSED_OVERLAY;
					dojo.addClass(icon, "closed");
					break;
				case Constants.IN_PROGRESS_STATES:	
					src = IMG_URL_IN_PROGRESS_OVERLAY;
					break;
			}
			if (src) {
				var overlayIcon = document.createElement("img");
				overlayIcon.className = "overlayIcon";
				overlayIcon.src = src;
				iconContainer.appendChild(overlayIcon);
			}
		}
		iconCell.appendChild(iconContainer);
		columns.push(iconCell);
		
		if (this.getPreference("secondIcon") != "none") {
			titleColSpan = 1;
			var secondIconCell = document.createElement("td");
			secondIconCell.className = "iconCell";
			var secondIcon = document.createElement("img");
			secondIcon.width = 16;
			secondIcon.height = 16;
			secondIcon.style.marginRight = "0.2em";
			var secondIconSrc;
			if (this.getPreference("secondIcon") == "priority")
				secondIconSrc = resultDTO.iconUrls[this._resultHeaderIndex[WorkItemAttributes.PRIORITY]];
			else if (this.getPreference("secondIcon") == "severity")
				secondIconSrc = resultDTO.iconUrls[this._resultHeaderIndex[WorkItemAttributes.SEVERITY]];
			else if (this.getPreference("secondIcon") == "status")
				secondIconSrc = resultDTO.iconUrls[this._resultHeaderIndex[WorkItemAttributes.STATE]];
			else if (this.getPreference("secondIcon") == "resolution")
				secondIconSrc = resultDTO.iconUrls[this._resultHeaderIndex[WorkItemAttributes.RESOLUTION]];
			if (secondIconSrc) {
				secondIcon.src = secondIconSrc;
				secondIconCell.appendChild(secondIcon);
			}
			columns.push(secondIconCell);
		}
		
		var titleCell = document.createElement("td");
		titleCell.className = "titleCell";
		titleCell.colSpan = titleColSpan;
		var titleWrapper = document.createElement("div");
		titleWrapper.className = "titleWrapper";
		var titleLink = document.createElement("a");
		titleLink.href = href;
		titleLink.innerHTML = resultDTO.id + ": " + FormatUtil.insertWordBreaks(resultDTO.labels[this._resultHeaderIndex[WorkItemAttributes.SUMMARY]], 20, true);
		titleWrapper.appendChild(titleLink);
		titleCell.appendChild(titleWrapper);
		columns.push(titleCell);
		
		return columns;
	},
	
	/* Node[] */ renderDetails: function(/* object */ resultDTO) {
		var columns = new Array();
		
		if (!this._detailsWidgets[resultDTO.id]) {
			this._detailsWidgets[resultDTO.id] = new com.ibm.team.dashboard.viewlets.web.ui.internal.QueryResultDetails({
				resultDTO: resultDTO,
				viewlet: this,
				headers: this._resultHeaders,
				headerIndex: this._resultHeaderIndex
			});
			this._detailsWidgets[resultDTO.id].populate();
		}
		var detailsCell = document.createElement("td");
		detailsCell.className = "detailsCell";
		detailsCell.colSpan = "3";
		detailsCell.appendChild(this._detailsWidgets[resultDTO.id].domNode);
		tr = document.createElement("tr");
		columns.push(detailsCell);
		
		return columns;
	},
	
	settingsChanged: function(/* object */ oldSettings, /* object */ newSettings) {
		var updateTitle = oldSettings.userTitle == true && newSettings.userTitle == false;
		if (oldSettings.preferences.query != newSettings.preferences.query) {
			this.reset(true);
			this.getSite().refresh();
			updateTitle = true;
		}
		else if (oldSettings.preferences.items != newSettings.preferences.items) {
			this.reset(false);
			this.update();
		}
		else if (oldSettings.preferences.secondIcon != newSettings.preferences.secondIcon) {
			this.update();
		}
		// getTextLabel is not CustomPreferenceProvider API
		// but is provided by the WorkItemQueryPreferenceProvider
		if (updateTitle)
			this.getSite().setTitle(this.getPreferenceProvider("query").getTextLabel(newSettings.preferences.query), true);
	},
	
	getTitleHyperlinkURL: function(){
		var queryId = this.getPreference("query");
		if (queryId != "-1")
			return WorkItemUtil.getRunSavedQueryURI(queryId);
		if (this._defaultQueryItemId)
			return WorkItemUtil.getRunSavedQueryURI(this._defaultQueryItemId);
		return null;
	},
	
	getPreferenceProvider: function(/* string */ preferenceId) {
		if (!this._preferenceProvider)
			this._preferenceProvider = new WorkItemQueryPreferenceProvider({ prefId: preferenceId });
		return this._preferenceProvider;
	},
	
	destroy: function() {
		for (var i in this._detailsWidgets)
			this._detailsWidgets[i].destroy();
		this.inherited(arguments);
	}
	
});

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.QueryResultDetails",
		[dijit._Widget, dijit._Templated], {
	
	templateString: "<div><div class=\"topDetailsSection\" dojoAttachPoint=\"_topSection\"></div><div class=\"detailsArea\"><div class=\"detailsColumn firstColumn\" dojoAttachPoint=\"_columnOne\"></div><div class=\"detailsColumn\" dojoAttachPoint=\"_columnTwo\"></div><div class=\"bottomDetailsSection\" dojoAttachPoint=\"_bottomSection\"></div><div class=\"descriptionLink\"><a href=\"#\" class=\"showDescription\" dojoAttachPoint=\"_descriptionLink\"><div></div><span>${showMore}</span></a></div><div dojoAttachPoint=\"_descriptionDiv\" class=\"loading hidden descriptionDiv\">${loadingMessage}</div></div></div>",
	
	constructor: function(args) {
		this._viewlet = args.viewlet;
		this._summaryDTO = args.resultDTO;
		this._headers = args.headers;
		this._headerIndex = args.headerIndex;
		this.loadingMessage = this._viewlet.messages.loading;
		this.showMore = this._viewlet.messages.showMore;
	},
	
	populate: function() {
		var status = new Array();
		status.push({value: this._summaryDTO.labels[this._headerIndex[WorkItemAttributes.STATE]], icon: this._summaryDTO.iconUrls[this._headerIndex[WorkItemAttributes.STATE]]});
		// TODO: This if will break under non-english since the label is being tested for "Unresolved".
		if (this._summaryDTO.labels[this._headerIndex[WorkItemAttributes.RESOLUTION]] && this._summaryDTO.labels[this._headerIndex[WorkItemAttributes.RESOLUTION]] != "Unresolved")
			status.push({value: this._summaryDTO.labels[this._headerIndex[WorkItemAttributes.RESOLUTION]], icon: this._summaryDTO.iconUrls[this._headerIndex[WorkItemAttributes.RESOLUTION]]});
		this._topSection.appendChild(this._createRow(null, status));
		this._columnOne.appendChild(
			this._createRow(this._headers[this._headerIndex[WorkItemAttributes.TYPE]].label,
			[{value: this._summaryDTO.labels[this._headerIndex[WorkItemAttributes.TYPE]],
			icon: this._summaryDTO.iconUrls[this._headerIndex[WorkItemAttributes.TYPE]]}])
			);
		this._columnOne.appendChild(
			this._createRow(this._headers[this._headerIndex[WorkItemAttributes.SEVERITY]].label,
			[{value: this._summaryDTO.labels[this._headerIndex[WorkItemAttributes.SEVERITY]],
			icon: this._summaryDTO.iconUrls[this._headerIndex[WorkItemAttributes.SEVERITY]]}])
			);
		var creationDate = new Date(parseInt(this._summaryDTO.labels[this._headerIndex[WorkItemAttributes.CREATION_DATE]]));
		creationDate = dojo.date.locale.format(creationDate, { selector: "date", formatLength: "medium" }) + " " + dojo.date.locale.format(creationDate, { selector: "time", formatLength: "short" });
		this._columnOne.appendChild(
			this._createRow(this._headers[this._headerIndex[WorkItemAttributes.CREATION_DATE]].label,
			[{value: creationDate,
			icon: this._summaryDTO.iconUrls[this._headerIndex[WorkItemAttributes.CREATION_DATE]]}])
			);
		this._columnOne.appendChild(
			this._createRow(this._headers[this._headerIndex[WorkItemAttributes.CREATOR]].label,
			[{value: this._summaryDTO.labels[this._headerIndex[WorkItemAttributes.CREATOR]],
			icon: this._summaryDTO.iconUrls[this._headerIndex[WorkItemAttributes.CREATOR]]}])
			);
		this._columnOne.appendChild(
			this._createRow(this._headers[this._headerIndex[WorkItemAttributes.PROJECT_AREA]].label,
			[{value: this._summaryDTO.labels[this._headerIndex[WorkItemAttributes.PROJECT_AREA]],
			icon: this._summaryDTO.iconUrls[this._headerIndex[WorkItemAttributes.PROJECT_AREA]]}])
			);
		this._columnTwo.appendChild(
			this._createRow(this._headers[this._headerIndex[WorkItemAttributes.CATEGORY]].label,
			[{value: this._summaryDTO.labels[this._headerIndex[WorkItemAttributes.CATEGORY]],
			icon: this._summaryDTO.iconUrls[this._headerIndex[WorkItemAttributes.CATEGORY]]}])
			);
		this._columnTwo.appendChild(
			this._createRow(this._headers[this._headerIndex[WorkItemAttributes.PRIORITY]].label,
			[{value: this._summaryDTO.labels[this._headerIndex[WorkItemAttributes.PRIORITY]],
			icon: this._summaryDTO.iconUrls[this._headerIndex[WorkItemAttributes.PRIORITY]]}])
			);
		this._columnTwo.appendChild(
			this._createRow(this._headers[this._headerIndex[WorkItemAttributes.TARGET]].label,
			[{value: this._summaryDTO.labels[this._headerIndex[WorkItemAttributes.TARGET]],
			icon: this._summaryDTO.iconUrls[this._headerIndex[WorkItemAttributes.TARGET]]}])
			);
		this._columnTwo.appendChild(
			this._createRow(this._headers[this._headerIndex[WorkItemAttributes.ESTIMATE]].label,
			[{value: this._summaryDTO.labels[this._headerIndex[WorkItemAttributes.ESTIMATE]],
			icon: this._summaryDTO.iconUrls[this._headerIndex[WorkItemAttributes.ESTIMATE]]}])
			);
		this._columnTwo.appendChild(
			this._createRow(this._headers[this._headerIndex[WorkItemAttributes.OWNER]].label,
			[{value: this._summaryDTO.labels[this._headerIndex[WorkItemAttributes.OWNER]],
			icon: this._summaryDTO.iconUrls[this._headerIndex[WorkItemAttributes.OWNER]]}])
			);
		if (this._summaryDTO.labels[this._headerIndex[WorkItemAttributes.TAGS]]) {
			this._bottomSection.appendChild(
				this._createRow(this._headers[this._headerIndex[WorkItemAttributes.TAGS]].label,
				[{value: this._summaryDTO.labels[this._headerIndex[WorkItemAttributes.TAGS]],
				icon: this._summaryDTO.iconUrls[this._headerIndex[WorkItemAttributes.TAGS]]}])
				);
		}
		this.connect(this._descriptionLink,"onclick",this._handleDescriptionLinkClick);
	},
	
	_createRow: function(name, values) {
		var wrapper = document.createElement("div");
		wrapper.className = "detailsRow";
		
		if (name) {
			var labelCell = document.createElement("div");
			labelCell.className = "detailsLabelCell";
			labelCell.appendChild(document.createTextNode(name + ":"));
			wrapper.appendChild(labelCell);
		}
		
		valueCell = document.createElement("div");
		valueCell.className = "detailsValueCell";
		for (var i = 0; i < values.length; i++) {
			if (values[i].icon) {
				var div = document.createElement("div");
				div.className = "detailsImageWrapper"
				var img = document.createElement("img");
				img.className = "detailsIcon";
				img.src = values[i].icon;
				div.appendChild(img);
				valueCell.appendChild(div);
			}
			var div = document.createElement("div");
			div.appendChild(document.createTextNode(values[i].value + (i != values.length-1 ? " - " : "")))
			valueCell.appendChild(div);
		}
		wrapper.appendChild(valueCell);
		return wrapper;
	},
	
	_handleDescriptionLinkClick: function (event) {
		dojo.stopEvent(event);
		
		var link = event.target.parentNode;
		this._setLinkShowHide(link,link.className != "showDescription");
		
		if (!this._workItemDTO)
			this._getWorkItemDTO();
	},
	
	_setLinkShowHide: function (/* domNode */ link, /* boolean */ show) {
		var span = link.lastChild;
		if (show) {
			span.innerHTML = this._viewlet.messages.showMore;
			link.className = "showDescription";
			dojo.addClass(this._descriptionDiv,"hidden");
		} else {
			span.innerHTML = this._viewlet.messages.showLess;
			link.className = "hideDescription";
			dojo.removeClass(this._descriptionDiv,"hidden");
		}
	},
	
	_getWorkItemDTO: function() {
		this._viewlet.getSite().setLoading(true);
		var self = this;
		var serviceObject = {
			success: function (result) {
				self._workItemDTO = result;
				self._populateDescription(result.attributes.description, result.attributes.internalComments ? result.attributes.internalComments.items : null);
				dojo.removeClass(self._descriptionDiv,"loading");
				self._viewlet.getSite().setLoading(false);
			},
			failure: function (error) {
				self._viewlet.getSite().setLoading(false);
			}
		};
		var responseHandler = new ServiceResponseHandler(serviceObject, "success", "failure");
		WorkItemClient.getWorkItem(responseHandler, {itemId: this._summaryDTO.itemId});
	},
	
	_populateDescription: function(input, comments) {
		while (this._descriptionDiv.firstChild)
			this._descriptionDiv.removeChild(this._descriptionDiv.firstChild);
		var descLabel = document.createElement("div");
		descLabel.className = "showMoreLabel";
		this._descriptionDiv.appendChild(descLabel);
		if (input && input.content) {
			descLabel.appendChild(document.createTextNode(this._viewlet.messages.description));
			this._descriptionDiv.appendChild(com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil.getSafeTextSpan(input.content));
			this._descriptionDiv.appendChild(document.createElement("br"));
		} else {
			descLabel.appendChild(document.createTextNode(this._viewlet.messages.noDescription));
		}
		var commentLabel = document.createElement("div");
		commentLabel.className = "showMoreLabel";
		this._descriptionDiv.appendChild(commentLabel);
		if (!comments || comments.length == 0) 
			commentLabel.appendChild(document.createTextNode(this._viewlet.messages.noDiscussion));
		else {
			commentLabel.appendChild(document.createTextNode(this._viewlet.messages.discussion));
			for (var i = 0; i < comments.length && i < 3; i++) {
				comments[i].commentNumber = i + 1;
				commentWidget = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemComment({
					comment: comments[i],
					editorUtil: com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil,
					workItem: this._workItemDTO,
					parentController: this
				});
				comments[i].commentWidget = commentWidget;
				commentWidget.collapse();
				this._descriptionDiv.appendChild(commentWidget.domNode);
			}
			if (comments.length > 3) {
				var more = document.createElement("a");
				more.href = WorkItemUtil.getViewWorkItemURI(this._summaryDTO.id);
				more.appendChild(document.createTextNode(this._viewlet.messages.goTo));
				this._descriptionDiv.appendChild(more);
			}
		}
	},
	
	getAttachmentInfo: function(number ) {
		if(!number) return;
		if(!isNaN(number)) {
			try{
				number = parseInt(number);
			} catch(e) { return };
		}
		if(!this._workItemDTO || !this._workItemDTO.linkTypes ||  this._workItemDTO.linkTypes.length == 0) return;
		for(var i=0;i<this._workItemDTO.linkTypes.length;i++) {
			if(this._workItemDTO.linkTypes[i].id=="com.ibm.team.workitem.linktype.attachment") {
				if(this._workItemDTO.linkTypes[i].linkDTOs) {
					for(j=0;j< this._workItemDTO.linkTypes[i].linkDTOs.length;j++) {
						if(this._workItemDTO.linkTypes[i].linkDTOs[j] 
							&& this._workItemDTO.linkTypes[i].linkDTOs[j].target 
							&&  this._workItemDTO.linkTypes[i].linkDTOs[j].target.id === number) {
							return this._workItemDTO.linkTypes[i].linkDTOs[j].target;
						}
					}
				}					
			}
		}
		
	}
	
});

})();
