/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.process_templates.ImportTemplateDialog"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dojox.data.dom");//$NON-NLS-1$
dojo.require("dojo.io.iframe");
dojo.require("dijit.ProgressBar"); //$NON-NLS-1$
dojo.require("dijit.form.CheckBox"); //$NON-NLS-1$

dojo.require("net.jazz.ajax.ui.internal.Dialog");//$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.client.internal.userRegistry");//$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");//$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.process_templates", "Messages");//$NON-NLS-1$//$NON-NLS-2$



(function(){

    var KEY_ESCAPE = 27;
    var KEY_RETURN = 13;
    var KEY_ENTER = 14;
    var KEY_SPACE = 32;
    
    var ImportTemplateDialogMessages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.process_templates", "Messages");//$NON-NLS-1$ //$NON-NLS-2$
    
    var DEFAULT_INTRO_TEXT = ImportTemplateDialogMessages.ImportTemplateDialog_1;
    var UserRegistryClient = com.ibm.team.repository.web.client.internal.userRegistry;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
    var SERVICE_URL = TeamServerClient._getServicePrefix() + "com.ibm.team.process.internal.service.web.IProcessDefinitionFileService/";//$NON-NLS-1$
    
    var _uploadFileInput_onchange_handle;
    var _uploadFileInput;
    var _okButton_onclick_handle;
    var _idPrompt_onchange_handle;
    var _newId_onchange_handle;
    var _idUpdate1_onclick_handle;
    var _idUpdate2_onclick_handle;
    var _jsonResponse;
    
    
    dojo.declare("com.ibm.team.process.web.ui.internal.admin.process_templates.ImportTemplateDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
    
        templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/process_templates/templates/ImportTemplateDialog.html"),//$NON-NLS-1$ //$NON-NLS-2$ 
        widgetsInTemplate: true,
        constructor: function(args){
            if (args == null) {
                args = new Object();
            }
            this.width = args.width || "450px"; //$NON-NLS-1$
            var properties = {
                "width": this.width //$NON-NLS-1$
            };
            this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.process_templates", "Messages");//$NON-NLS-1$ //$NON-NLS-2$
        },
        
        postCreate: function(){
            this.dialog = new net.jazz.ajax.ui.internal.Dialog({
                contentNode: this.contentNode,
                primaryTitle: this.messages.ImportTemplateDialog_2,
                width: this.width,
                isClosable: this.isClosable
            });
            this._topText.innerHTML = DEFAULT_INTRO_TEXT;
            
            var div = this._uploadInputDiv;
            dojox.data.dom.removeChildren(div);
            var input1 = document.createElement("input"); //$NON-NLS-1$
            input1.type = "file"; //$NON-NLS-1$
            input1.name = "uploadFileInput";//$NON-NLS-1$
            input1.size = 43;
            this._uploadFileInput = input1;
            this._uploadFileInput_onchange_handle = dojo.connect(this._uploadFileInput, "onchange", this, "_handleDirectoryChange"); //$NON-NLS-1$ //$NON-NLS-2$
			if (!dojo.isIE) {
				// setting focus here causes IE to not show the field, so don't 
				//do it 
            	this._uploadFileInput.focus();
            }
            div.appendChild(input1);
            
            this.connect(this.cancelButton, "onclick", function(e){ //$NON-NLS-1$
                dojo.stopEvent(e);
                this.dialog.close();
            });
            this._okButton_onclick_handle = this.connect(this.okButton, "onclick", "_onOk"); //$NON-NLS-1$ //$NON-NLS-2$ 
            
        },
        /**
         * enable the OK button if the file is a zip
         * @param e is an event
         */ 
        _handleDirectoryChange: function(e){
            if (e != null) {
                dojo.stopEvent(e);
            }
            
            this._clearStatusMessage();
            var fullPath = this._uploadFileInput.value;
            if (fullPath.length != 0) {
                var parts = fullPath.split("\\"); //$NON-NLS-1$
                var fileName = parts[parts.length - 1];
                var extensionIndex = fileName.lastIndexOf(".zip"); //$NON-NLS-1$
                if (!(extensionIndex == fileName.length - 4)) {
                    this._setStatusMessage(this.messages.ImportTemplateDialog_3, true);
                    return;
                }
                else {
                    this.okButton.disabled = false;
                }
            }
        },
        _runSelect: function(e){
            dojo.stopEvent(e);
            
        },        
        _runDeselect: function(e){
            dojo.stopEvent(e);
            
        },        
        _onOk: function(e){
            if (e != null) {
                dojo.stopEvent(e);
            }
            this._setStatusMessage(this.messages.ImportTemplateDialog_4);
            this._showProgressBar();
            //save "this" reference so it can be used in callback from send
            var self = this;
            dojo.io.iframe.send({
                form: this._uploadForm,
                handleAs: "json",//$NON-NLS-1$
                url: SERVICE_URL,
                handle: function(response, ioArgs){
                    self._handleLoad(response, ioArgs);
                },
                error: function(response, ioArgs){
                    self._handleErrorOnUpload(response, ioArgs);
                }
            });
            
        },
        _handleLoad: function(response, ioArgs){
            this._jsonResponse = dojo.fromJson(response);
            switch (this._jsonResponse.status) {
				case 200:
				//success case - template overwritten
				this.onSuccess(this._jsonResponse.processId, false);
				break;
                case 201:
                //success case - template created
                this.onSuccess(this._jsonResponse.processId, true);
                break;
                case 409:
                    //409 indicates the id already exists.  prompt the user to
                    //overwrite or enter a new id.
                    this._handleDuplicateId();
                    break;
                case 410:
                    //zip file not on server
                    this._handleFileGone();
                    break;
                default:
                    //any other error
                    this._setStatusMessage(this._jsonResponse.message, true);
                    this._progress.domNode.style.display = "none"; //$NON-NLS-1$
                    this.okButton.disabled = true;
                    break;
            }
        },
		onSuccess: function (processId, isNew) {
			this.dialog.close();
		},
        _handleDuplicateId: function(){
            this._hideFileUploadSection();
            
            var iconPath = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/warning.gif"); //$NON-NLS-1$ //$NON-NLS-2$ 
            var imgHtml = "<img src=\"" + iconPath + "\"\/>"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //the last NLS should not be needed, but extra quotes throw off the builder.
            var dupHeader = this.messages.ImportTemplateDialog_11; 
            var brHtml = "<br />"; //$NON-NLS-1$
            var dupMessage = dojo.string.substitute(this.messages.ImportTemplateDialog_6, [this._jsonResponse.processId]);
            this._topText.innerHTML =  imgHtml + dupHeader + brHtml + dupMessage;
            
            this._showProcessIdPrompt();
            if (!this._idUpdate1.checked && !this._idUpdate2.checked) {
				this._idUpdate2.setChecked (true);
			}
            this._newId.value = this._jsonResponse.processId;
            this._newId.disabled = !this._idUpdate2.checked;
			this._newName.value = this._jsonResponse.name;
			this._newName.disabled = !this._idUpdate2.checked;
			
            this.okButton.disabled = true;
            
            this.disconnect(this._okButton_onclick_handle);
            this._okButton_onclick_handle = this.connect(this.okButton, "onclick", "_onOkDuplicateId"); //$NON-NLS-1$ //$NON-NLS-2$
            //connect to onClick (not onclick) because this is a dijit.
            this._idUpdate1_onclick_handle = this.connect(this._idUpdate1, "onClick", "_handleChooseOverwrite");//$NON-NLS-1$ //$NON-NLS-2$
            this._idUpdate2_onclick_handle = this.connect(this._idUpdate2, "onClick", "_handleChooseChangeId"); //$NON-NLS-1$ //$NON-NLS-2$
            this._newId_onchange_handle = dojo.connect(this._newId, "onkeyup", this, "_handleIdChange");//$NON-NLS-1$ //$NON-NLS-2$
			
        },
        _handleFileGone: function(){
            //show upload file
            this._topText.innerHTML = this.DEFAULT_INTRO_TEXT;
            this._showFileUploadSection();
            this.disconnect(this._okButton_onclick_handle);
            this._okButton_onclick_handle = this.connect(this.okButton, "onclick", "_onOk");//$NON-NLS-1$ //$NON-NLS-2$
        },
        /* 
         * enable the OK button if the user chooses to overwrite
         */
        _handleChooseOverwrite: function(e){
            this.okButton.disabled = false;
            this._newId.disabled = true;
			this._newName.disabled = true;
        },
        /* 
         * enable the OK button if the user chooses a new ID
         */
        _handleChooseChangeId: function(e){
            this.okButton.disabled = (this._newId.value === this._jsonResponse.processId);
            this._newId.disabled = false;
			this._newName.disabled = false;
        },
        /*
         * Post the id and overwrite value to the service to save the template
         * when the user clicks OK after getting a duplicate ID error.
         */
        _onOkDuplicateId: function(e){
			this._showProgressBar();
			
            var newId = this._newId.value;
            newId= newId.replace(/%/g, "%25"); //$NON-NLS-1$ //$NON-NLS-2$
			var newName = this._newName.value;
			newName = newName.replace(/%/g, "%25"); //$NON-NLS-1$ //$NON-NLS-2$
            var key = this._jsonResponse.key;
            var overwrite = this._idUpdate1.checked;
			var theUrl;
			var theStatusMessage;
			if (overwrite) {
				theUrl = SERVICE_URL + key + "?overwrite=" + overwrite; //$NON-NLS-1$
				theStatusMessage = this.messages.ImportTemplateDialog_7;
			} else {
				theUrl = SERVICE_URL + key + "?processId=" + newId  + "&name=" + newName; //$NON-NLS-1$ //$NON-NLS-2$
				theStatusMessage = this.messages.ImportTemplateDialog_8;
			}
			this._setStatusMessage(theStatusMessage, false);
			
			var self = this;
            //use iframe transport here so that both calls (initial and this one
            //can use the same response handling
            dojo.io.iframe.send({
                form: this._overwriteOrChangeIdForm,
                handleAs: "json",//$NON-NLS-1$
                url: theUrl,
                handle: function(response, ioArgs){
                    self._handleLoad(response, ioArgs);
                },
                error: function(response, ioArgs){
                    self._handleErrorOnUpload(response, ioArgs);
                }
            });
        },
        
        _handleErrorOnUpload: function(response, ioArgs){
            this._setStatusMessage(response, true);
        },
        
        _hideFileUploadSection: function(){
            this._clearStatusMessage();
            this._uploadForm.style.display = "none"; //$NON-NLS-1$
            this._progress.domNode.style.display = "none"; //$NON-NLS-1$
        },
        
        _showFileUploadSection: function(){
            this._uploadForm.style.display = "block";//$NON-NLS-1$
        },
        
        _showProcessIdPrompt: function(){
            this._idPrompt.style.display = "block";//$NON-NLS-1$
        },
        
        _handleIdChange: function(e){
            if (this._newId && this._newId.value.length > 0) {
                this.okButton.disabled = false;
            }
            else {
                this.okButton.disabled = true;
            }
        },
        _queryError: function(err){
            this._setStatusMessage(err.message, true);
        },
        
        _clearStatusMessage: function(){
            this.status.style.visibility = "hidden";//$NON-NLS-1$
        },
        
        _setStatusMessage: function(msg, isError){
            this.status.style.visibility = "visible";//$NON-NLS-1$
            dojox.data.dom.replaceChildren(this.status, document.createTextNode(msg));
            this.status.className = isError ? "ErrorStatus" : "NormalStatus";//$NON-NLS-1$ //$NON-NLS-2$
        },
        
        _showProgressBar: function(){          
            this._progress.domNode.style.display = "block";//$NON-NLS-1$
        },
        
        destroyWidget: function(){
            if (this._uploadFileInput_onchange_handle) {
                dojo.disconnect(this._uploadFileInput_onchange_handle);
            }
            if (this._idPrompt_onchange_handle) {
                dojo.disconnect(this._idPrompt_onchange_handle);
            }
            if (this._okButton_onclick_handle) {
                dojo.disconnect(this._okButton_onclick_handle);
            }
            if (this._newId_onchange_handle) {
                dojo.disconnect(this._newId_onchange_handle);
            }
            this.destroy();
        }
    });
    
})();
