/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.dnd.DraggableContainer");

dojo.require("com.ibm.team.dashboard.web.ui.internal.dnd.DraggableManager");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("dojo.dnd.common");
dojo.require("dojo.dnd.autoscroll");

(function() {
	var draggableManager = com.ibm.team.dashboard.web.ui.internal.dnd.draggableManager;
	var Util = com.ibm.team.dashboard.web.util.internal.Util;
	
	dojo.declare("com.ibm.team.dashboard.web.ui.internal.dnd.DraggableContainer", null, {
		WIDGET_TOP_MARGIN: 0,
		WIDGET_BOTTOM_MARGIN: 0,
		WIDGET_LEFT_MARGIN: 0,
		WIDGET_RIGHT_MARGIN: 0,
		WIDGET_WIDTH_PADDING: 0,
		constructor: function(node, params){
			this.index = params.index;
			if (!this.index) {
				this.index = dojo.dnd.getUniqueId();
			}
			this.node = node;
			
			this.accept = params.accept;
			
			this.widgets = new Array();
			this.overWidget = null;
			
			this.isSource = true;
			
			this.events = [];
			this.events = new Array();
			this.events.push(dojo.connect(node, "ondragstart", this, this.checkDrag));
			this.events.push(dojo.connect(node, "onselectstart", this, this.checkDrag));
			this.events.push(dojo.connect(document.body,"ondragstart", this, this.checkDrag));
			this.events.push(dojo.connect(document.body,"onselectstart", this, this.checkDrag));
			
			dojo.addClass(this.node,"dndContainer");
			
			this.registerContainer();
		},
		registerContainer: function() {
			draggableManager().containers.push(this);
		},
		//If we're dragging a viewlet, cancel text select
		checkDrag: function(event) {
			if (draggableManager().isDragging || draggableManager().fromContainer) {
				dojo.stopEvent(event);
			}
		},
		/*
		 * Widget:
		 * 	dojo widget, or domNode wrapped in an object literal
		 * 	{domNode: domNode}
		 * Params:
		 * 	index: insertion index
		 * 	handle: domNode that initiates drag (if null, defaults to whole widget)
		 */
		add: function(widget, params) {
			var index = params.index;
			if (typeof(index) != "number" || index >= this.widgets.length) {
				index = this.widgets.length;
			}
			widget.type = params.type;
			widget.index = index;
			widget.handle = params.handle;
			if (!widget.handle) {
				widget.handle = widget.domNode;
			}
			if (index == this.widgets.length) {
				if (!this.node.lastChild || dojo.hasClass(this.node.lastChild,"dndItem")){
					this.node.appendChild(widget.domNode);
				} else {
					this.node.insertBefore(widget.domNode,this.node.lastChild);
				}	
			} else {
				this.node.insertBefore(widget.domNode, this.widgets[index].domNode);
			}
			this.widgets.splice(index,0,widget);
			widget.dndEvent = dojo.connect(widget.handle,"onmousedown",this,this.onMouseDown);
			
			dojo.addClass(widget.domNode,"dndItem");
			dojo.addClass(widget.handle,"dndHandle");
			if (params.animate) {
				widget.domNode.style.height ="1px";
				dojo._setOpacity(widget.domNode,0);
				var args = {node: widget.domNode, duration: 300};
				var animation = dojo.fx.chain([dojo.fx.wipeIn(args), dojo.fadeIn(args)]);
				animation.play();
				dojo.connect(animation,"onEnd",widget,function() {
					this.domNode.style.overflow = "";
				});
			}
			this.updateIndices();
		},
		destroy: function() {
			dojo.removeClass(this.node,"dndContainer");
			while(this.events && this.events.length > 0) {
				dojo.disconnect(this.events.pop());
			}
			while (this.widgets && this.widgets.length > 0) {
				var widget = this.widgets.pop();
				dojo.disconnect(widget.dndEvent);
				if (widget.destroy) {
					widget.destroy();	
				}
			}
			var containers = draggableManager().containers;
			for (var i = 0; i < containers.length; i++) {
				if (containers[i] == this) {
					containers.splice(i,1);
				}
			}
		},
		move: function(fromIndex, toContainerIndex, toIndex) {
			var fromContainer = draggableManager().fromContainer;
			var overContainer = draggableManager().overContainer;
			
			var widget = fromContainer.widgets.splice(fromIndex,1)[0];
			widget.domNode.parentNode.removeChild(widget.domNode);
			overContainer.node.insertBefore(widget.domNode, this.getPlaceholder());
			overContainer.widgets.splice(toIndex,0,widget);
			
			widget._columnIndex = toContainerIndex;
			
			var cols = [fromContainer, overContainer];
			for (var j = 0; j < cols.length; j++) {
				this.updateIndices(cols[j]);
				if (fromContainer == overContainer) break;
			}
		},
		remove: function(index){
			//remove widget from array
			var widget = this.widgets.splice(index,1)[0];

			// destroy widget
			if (widget.destroy) {
				widget.destroy();	
			}
			this.updateIndices();
		},
		onMouseDown: function(event) {
			if (this._disabled) {
				return;
			}
			for (var i = 0; i < this.widgets.length; i++) {
				if (event.currentTarget == this.widgets[i].handle) {
					var widget = this.widgets[i];
					break;
				}
			}
			var coords = dojo.coords(event.currentTarget, true);
			draggableManager().initialX = event.pageX;
			draggableManager().initialY = event.pageY;
			draggableManager().offsetX = event.pageX - coords.x;
			draggableManager().offsetY = event.pageY - coords.y;
			draggableManager().widgetCoords = coords;
			if (widget) {
				this.onDndStart(widget);
				this.overWidget = widget;
			}
			this.checkDrag(event);
		},
		onMouseMove: function(event) {
			com.ibm.team.dashboard.web.ui.internal.dnd.autoScroll(event);
			this.dropAtEnd = false;
			var x = event.pageX;
			var y = event.pageY;
			for (var i = 0; i < this.widgets.length; i++) {
				var overCoords = {
					xy: dojo.coords(this.widgets[i].domNode, true),
					w: this.widgets[i].domNode.offsetWidth,
					h: this.widgets[i].domNode.offsetHeight
				};
				if ((this.constrainAvatar && this.constrainAvatar == "Y") || (x >= overCoords.xy.x && x <= overCoords.xy.x + overCoords.w && this.widgets[i] != draggableManager().widget)) {
					if (this.constrainAvatar && this.constrainAvatar == "A")
						var row = Math.round((overCoords.xy.y - dojo.style(this.widgets[i].domNode, "marginTop") - draggableManager().containerCoords.y) / 27);
					if ((this.constrainAvatar && this.constrainAvatar == "X") || (y >= overCoords.xy.y && y <= overCoords.xy.y + overCoords.h) || (this.constrainAvatar && this.constrainAvatar == "A" && ((row == 0 && y < overCoords.xy.y) || (row == draggableManager().maxRow && y > overCoords.xy.y + overCoords.h)))) {
						if (this.horizontal) {
							var coords = dojo.coords(draggableManager().avatar.node, true);
							if (draggableManager().atStart) {
								var before = true;
							} else if (draggableManager().atRowEnd) {
								var before = false;
							} else {
								var before = (coords.x + coords.w / 2 - overCoords.xy.x) < (overCoords.w / 2);	
							}
						} else {
							var before = (event.pageY - overCoords.xy.y) < (overCoords.h / 2);
						} 
						 
						draggableManager().before = before;
						if (this.overWidget != this.widgets[i]) {
							this.overWidget = this.widgets[i];
						}
						if (this.overWidget) {
							this.insertPlaceholder(before);
						}
						return;
					}
				}
			}
			var placeholder = this.getPlaceholder();
			if (placeholder) {
				var placeholderCoords = {
					xy: dojo.coords(placeholder, true),
					w: placeholder.offsetWidth,
					h: placeholder.offsetHeight + this.WIDGET_TOP_MARGIN
				};
			
				if ((this.constrainAvatar && this.constrainAvatar == "Y") || (x >= placeholderCoords.xy.x - this.WIDGET_LEFT_MARGIN && x <= placeholderCoords.xy.x + placeholderCoords.w)) {
					if ((this.constrainAvatar && this.constrainAvatar == "X") || (y >= placeholderCoords.xy.y - this.WIDGET_TOP_MARGIN && y <= placeholderCoords.xy.y + placeholderCoords.h)) {
						return;
					}
				}
			}
			if (!this.horizontal || draggableManager().atEnd) {
				this.dropAtEnd = true;
				this.insertPlaceholder(false);
				this.overWidget = this.widgets[this.widgets.length - 1];	
			}
		},
		onMouseUp: function(event) {
			if (draggableManager().isDragging) {
				var fromIndex = draggableManager().widget.index;
				if (draggableManager().overContainer.isSource) {
					var fromContainerIndex = draggableManager().fromContainer.index;
					var overContainerIndex = draggableManager().overContainer.index;
					var placeholder = this.getPlaceholder();
					if (placeholder){
						var toIndex = this.getPlaceholder().index;	
					} else {
						this.onDndCancel(false);
						return;
					}
					try {
						if (fromContainerIndex != overContainerIndex || fromIndex != toIndex) {
							this.move(fromIndex, overContainerIndex, toIndex);
						}	
					} catch (err) {
						this.onDndCancel(false);
						console.err("Drag and drop error: " + err);
					}		
				} else {
					var toTarget = draggableManager().overContainer;
					var cancelDrag = toTarget.move(fromIndex, toTarget.index);
					if (cancelDrag) {
						draggableManager().fromContainer.onDndCancel(false);
						return;
					}
				}
			}
			this.onDndCancel(true);
		},
		onDndStart: function(widget) {
			if (widget.dndEvent) {
				dojo.disconnect(widget.dndEvent);	
			}
			dojo.addClass(document.body,"moveCursor");
			widget.dndEvent = dojo.connect(document.body,"onmouseup",this,this.onMouseUp);
			draggableManager().initDrag(this,widget);
		},
		onDndCancel: function(success) {
			var widget = draggableManager().widget;
			dojo.removeClass(document.body,"moveCursor");
			if (widget) {
				if (widget.dndEvent) {
					dojo.disconnect(widget.dndEvent);
				}
				if (success) {
					container = draggableManager().overContainer;
				} else {
					container = this;
				}
				widget.dndEvent = dojo.connect(widget.handle,"onmousedown",container,this.onMouseDown);				
			}
			draggableManager().stopDrag();
			this.checkPlaceholder();
		},
		insertPlaceholder: function(before){
			this.height = draggableManager().avatar.node.offsetHeight;
			if (this.height == 0) {
				this.height = 50;
			}
			this.checkPlaceholder();
			
			var placeholder = document.createElement("div");
			placeholder.id = "draggable-placeholder";
			placeholder.style.height = this.height - (this.WIDGET_TOP_MARGIN + this.WIDGET_BOTTOM_MARGIN) + "px";
			if (this.horizontal) {
				placeholder.style.width = dojo.byId("avatar").offsetWidth - this.WIDGET_LEFT_MARGIN - this.WIDGET_RIGHT_MARGIN + "px";
			}
			var dec = 0;
			if (this == draggableManager().fromContainer && this.overWidget.index >= draggableManager().widget.index) {
				dec = 1;
			}
			
			if (this.dropAtEnd) {
				if (!this.node.lastChild || dojo.hasClass(this.node.lastChild,"dndItem")){
					this.node.appendChild(placeholder);
				} else {
					this.node.insertBefore(placeholder,this.node.lastChild);
				}
				placeholder.index = this.widgets.length - dec;
				return;
			}
			if (before) {
				this.node.insertBefore(placeholder,this.overWidget.domNode);
				placeholder.index = this.overWidget.index - dec;
			} else {
				var next = this.overWidget.domNode.nextSibling;
				if (!next) {
					this.node.appendChild(placeholder);
				} else {
					this.node.insertBefore(placeholder,next);
				}
				placeholder.index = this.overWidget.index + 1 - dec;
			}
		},	
		checkPlaceholder: function() {
			var checkPlaceholder = this.getPlaceholder();
			if (checkPlaceholder) {
				checkPlaceholder.parentNode.removeChild(checkPlaceholder);
			}
		},
		getPlaceholder: function() {
			return dojo.byId("draggable-placeholder");
		},
		updateIndices: function(col) {
			if (!col) {
				col = this;
			}
			for (var i = 0; i < col.widgets.length; i++) {
				col.widgets[i].index = i;
			}
		}
	});
	
	dojo.provide("com.ibm.team.dashboard.web.ui.internal.dnd.Avatar");	
	
	com.ibm.team.dashboard.web.ui.internal.dnd.Avatar = function(manager, className){
		this.manager = manager;
		this.construct(className);
	};
	
	dojo.extend(com.ibm.team.dashboard.web.ui.internal.dnd.Avatar, {
		construct: function(className) {
			var a = dojo.doc.createElement("div");
			a.className = "dndAvatar";
			var classes = className.split(" ");
			for (var i = 0; i < classes.length; i++) {
				if(classes[i]) {
					dojo.addClass(a,classes[i]);
				}
			}
			a.id="avatar";
			a.style.position = "absolute";
			a.style.zIndex = 1999;
			a.style.margin = "0px";
			a.style.width = "33%";
			var widget = this.manager.widget;
			var node = widget.domNode.cloneNode(true);
			a.style.width = widget.domNode.clientWidth + this.manager.fromContainer.WIDGET_WIDTH_PADDING + "px";
			widget.domNode.style.display = "none";
			node.id = "";
			a.appendChild(node);
			this.node = a;
		},
		destroy: function(){
			dojo._destroyElement(this.node);
			if (this.manager.widget.domNode) {
				this.manager.widget.domNode.style.display = "";	
			}
			this.node = false;
		}
	});
	
	com.ibm.team.dashboard.web.ui.internal.dnd.autoScroll = function(e){
		// summary: a handler for onmousemove event, which scrolls the window, if necesary
		// e: Event: onmousemove event
		var v = dojo.dnd.getViewport(), dx = 0, dy = 0;
		if(e.clientX < dojo.dnd.H_TRIGGER_AUTOSCROLL){
			dx = -dojo.dnd.H_AUTOSCROLL_VALUE;
		} else if(e.clientX > v.w - dojo.dnd.H_TRIGGER_AUTOSCROLL){
			if (document.documentElement.scrollLeft < draggableManager()._scrollMaxX) {
				dx = dojo.dnd.H_AUTOSCROLL_VALUE;
			}
		}
		if(e.clientY < dojo.dnd.V_TRIGGER_AUTOSCROLL){
			dy = -dojo.dnd.V_AUTOSCROLL_VALUE;
		} else if(e.clientY > v.h - dojo.dnd.V_TRIGGER_AUTOSCROLL){
			if (document.documentElement.scrollTop < draggableManager()._scrollMaxY) {
				dy = dojo.dnd.V_AUTOSCROLL_VALUE;
			}
		}
		window.scrollBy(dx, dy);
	};
})();