/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.dnd.ViewletColumn");

dojo.require("com.ibm.team.dashboard.web.ui.internal.dnd.DraggableContainer");
dojo.require("com.ibm.team.dashboard.web.ui.internal.dnd.DraggableManager");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("dojo.dnd.autoscroll");

(function() {
	var draggableManager = com.ibm.team.dashboard.web.ui.internal.dnd.draggableManager;
	var Util = com.ibm.team.dashboard.web.util.internal.Util;
	
	dojo.declare("com.ibm.team.dashboard.web.ui.internal.dnd.ViewletColumn", com.ibm.team.dashboard.web.ui.internal.dnd.DraggableContainer, {
		constructor: function(node, params){
			this.avatarClass = "viewletColumnAvatar com-ibm-team-dashboard-content";
			this._columnHelper = params.columnHelper;
			this._viewletPage = params.viewletPage;
			this._disabled = false;
			if (params.disabled) {
				this.setEditMode(!params.disabled);
			}
			this.index = params.columnIndex;
		},
		WIDGET_TOP_MARGIN: 10,
		WIDGET_BOTTOM_MARGIN: 10,
		WIDGET_WIDTH_PADDING: 14,
		checkDrag: function(event) {
			if (dojo.hasClass(event.target.parentNode,'header') || dojo.hasClass(event.target.parentNode.parentNode,'header')) {
				dojo.stopEvent(event);
			}
			this.inherited(arguments);
		},
		onMouseDown: function(event) {
			if (this._disabled) {
				return;
			}
			
			this.inherited(arguments);
			
			var widget = draggableManager().widget;
			if (widget && widget.isZoomed) {
				var preCoords = dojo.coords(event.currentTarget, true);
				var coords = dojo.coords(widget.zoomPlaceholder, true);
				var offsetX = event.pageX - preCoords.x;
				var offsetY = event.pageY - preCoords.y;  
				if (offsetX > coords.w - 80) offsetX = coords.w - 80;
				draggableManager().offsetX = offsetX;
				draggableManager().offsetY = offsetY;
			}
			draggableManager().offsetX = draggableManager().offsetX + 4;
		},
		setEditMode: function(editing) {
			this._disabled = !editing;
			for(var i = 0; i< this.widgets.length; i++) {
				if (this.widgets[i] && this.widgets[i].setEditMode) {
					this.widgets[i].setEditMode(editing);
					if (editing) {
						dojo.addClass(this.widgets[i].handle,"dndHandle");
					} else {
						dojo.removeClass(this.widgets[i].handle,"dndHandle");
					}
				}
			}
			if (editing) {
				dojo.removeClass(this.node,"dndContainerDisabled");
			} else {
				dojo.addClass(this.node,"dndContainerDisabled");
			}
		},
		add: function(widget, params) {
			this.inherited(arguments);
			if(this._disabled) {
				dojo.removeClass(widget.handle,"dndHandle");	
			}
		},
		move: function(fromIndex, toContainerIndex, toIndex) {
			try {
				var dashboardItem = this._columnHelper.getDashboardItem();
				dashboardItem.moveViewlet(draggableManager().widget._viewletHelper.getId(), null, toContainerIndex, toIndex);
			} catch (err) {
				console.error("New viewlet position could not be saved: " + err.message + " - Refreshing viewlet page.");
				this._viewletPage.refreshPage();
			}
			this.inherited(arguments);
		},
		remove: function(index){
			var widget = this.widgets[index];
			// save state
			this._columnHelper.getDashboardItem().removeViewlet(widget._viewletHelper.getId());
			this.inherited(arguments);
			this._viewletPage.checkEmpty();
		},
		onDndCancel: function(success) {
			this.inherited(arguments);
			dojo.publish("com.ibm.team.dashboard.web.onresize");
		}
	});
})();
