/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.util.ResizeNotifier");

(function() {
	com.ibm.team.dashboard.web.ui.internal.util.ResizeNotifier = function(){
		this._nodes = new Array();
		this._publishEvents = new Array();
		this._events = new Array();
		this._isInitialized = false;
		this._width = 0;
	};
	
	dojo.extend(com.ibm.team.dashboard.web.ui.internal.util.ResizeNotifier, {
		destroy: function() {
			this._nodes = new Array();
			while(this._publishEvents.length > 0)
				dojo.unsubscribe(this._publishEvents.pop());
			this._publishEvents = new Array();
			while(this._events.length > 0)
				dojo.disconnect(this._events.pop());
			this._events = new Array();
			this._isInitialized = false;
		},
		_init: function() {
			//Left-nav resize event
			this._publishEvents.push(dojo.subscribe("net.jazz.ajax.ui.SideNavigationLayout.slideLeftNav.onEnd",this,"_onLeftNavResize"));
			
			//Dashboard resize event (called when a scrollbar could be added - must check width to verify legitimacy)
			this._publishEvents.push(dojo.subscribe("com.ibm.team.dashboard.web.onresize",this,"_onResize"));
			
			//Window resize event
			this._events.push(dojo.connect(window,'onresize',this,this._onResize));
			
			this._width = document.documentElement.clientWidth;
			this._isInitialized = true;
		},
		unregister: function(id) {
			for(var i = 0; i < this._nodes.length; i++) {
				if (this._nodes[i].id == id) {
					this._nodes.splice(i,1);
					if (this._nodes.length == 0) this.destroy();
					return;
				}
			}
		},
		register: function(args) {
			if (!args.callback) return;
			if (typeof args.browserEvents == "undefined") args.browserEvents = true;
			if (typeof args.publishEvents == "undefined") args.publishEvents = true;
			args.id = this._getUniqueId();
			this._nodes.push(args);
			// if this was the first node registered
			if (!this._isInitialized)
				this._init();

			return args.id;
		},
		_getUniqueId: function() {
			return Date.parse(new Date().toString());
		},
		_onLeftNavResize: function() {
			//always notify nodes when left-nav changes
			this._onResize(null, true);
		},
		_onResize: function(event, leftNavResize) {
			for (var i = 0; i < this._nodes.length; i++) {
				if (leftNavResize || this._nodes[i].force || this._width != document.documentElement.clientWidth) {
					if (this._nodes[i] && (!event || this._nodes[i].browserEvents) && (event || this._nodes[i].publishEvents) && this._nodes[i].callback) {
						try {
							setTimeout(dojo.hitch(this._nodes[i].context, this._nodes[i].callback), 0);
						} catch (err) {
							console.err("Failed to notify client about resize event: " + err);
						}
					}
				}
			}
			this._width = document.documentElement.clientWidth;
		}
	});
	
	com.ibm.team.dashboard.web.ui.internal.util._resizeNotifier = null;
	
	com.ibm.team.dashboard.web.ui.internal.util.resizeNotifier = function(){
		if(!com.ibm.team.dashboard.web.ui.internal.util._resizeNotifier){
			com.ibm.team.dashboard.web.ui.internal.util._resizeNotifier = new com.ibm.team.dashboard.web.ui.internal.util.ResizeNotifier();
		}
		return com.ibm.team.dashboard.web.ui.internal.util._resizeNotifier;
	};
})();
