/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemComment");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("dojo.date");

dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.SelfSizingTextAreaWidget");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var WorkItemUtil            = com.ibm.team.workitem.web.client.util;
var EditorUtil = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;

/**
 * Top-level work item editor; has a persistent header (defined in this module)
 * and a set of child widgets (mapped in this._childWidgets).
 */
/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemComment",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/editor/templates/WorkItemComment.html"),
		// templateCssPath: dojo.uri.dojoUri("../com.ibm.team.workitem.web/ui/internal/view/editor/templates/WorkItemComment.css"),
		// populate the template with data, connect events, and show initial
		// child widget
		isNew: false,
        constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
		},	
		postCreate: function() {
			var self = this;
			this._commentHeader.title = this.messages.titleCommentCollapse;			
			this.commentTextArea = new com.ibm.team.workitem.web.ui.internal.view.layout.SelfSizingTextAreaWidget({parentController: this.parentController});
			dojo.addClass(this.commentTextArea.textarea, "SelfSizingTextAreaComment");
			this._newCommentArea.appendChild(this.commentTextArea.domNode);

			if(this.comment == null) {
				this._commentHeader.style.display="none";
				this._oldCommentArea.style.display="none";
				this.isNew = true;
				return;
			}
			 	
			this._bindCommentHeader(this.comment);
			this._bindCommentBody(this.comment);
			
		},
		destroyWidget: function() {
			this._stopNewCommentInterval();
			this.destroy();
		},
		_bindCommentHeader: function(comment) {
			// "<span class=\"commentNumber\">" + comment.commentNumber + ".</span>";
			var commentSpan = document.createElement("SPAN");
			commentSpan.className="commentNumber";
			commentSpan.appendChild(document.createTextNode(comment.commentNumber + "."));
			this._commentHeaderSPAN.appendChild(commentSpan);			
		
			var contribName= comment.creator.name;
			var contribEmail= comment.creator.emailAddress;
			
			// prevent scripts from running in the span
			contribEmail.replace("<", "&lt;");
			contribEmail.replace(">", "&gt;");
			contribName.replace("<", "&lt;");
			contribName.replace(">", "&gt;");

// "<span class=\"commentDetails\"><a title='Send an email to the user' href=\"mailto:" + contribEmail + "\">" + contribName + "</a> ";
			var emailSpan = document.createElement("SPAN");
			emailSpan.className="commentDetails";
			var emailAnchor = document.createElement("A");
			emailAnchor.title= title=this.messages.titleCommentSendEmail;
			emailAnchor.href= "mailto:" + contribEmail;
			emailAnchor.appendChild(document.createTextNode(contribName));
			emailSpan.appendChild(emailAnchor);
			
			this._commentHeaderSPAN.appendChild(emailSpan);
			
			
//			var commentDate= dojo.date.fromIso8601(comment.creationDate);
			var commentDateStr = "[not migrated]";
			var commentDate= WorkItemUtil.fromISOString(comment.creationDate);
			if(commentDate) {
				commentDateStr= this.editorUtil._getDateString(commentDate, true);
			}
			
			var dateSpan = document.createElement("SPAN");
			dateSpan.className="CommentDate";
			dateSpan.appendChild(document.createTextNode(commentDateStr));
					
			this._commentHeaderSPAN.appendChild(dateSpan);
			
		},

		_bindCommentBody: function(comment) {
            if(!comment) return;
			var span = EditorUtil.getSafeTextSpan(comment.content, this, comment.links);
			dojox.data.dom.replaceChildren(this._commentBodySPAN,span);
			EditorUtil.linkAttachmentPreview(span, this.workItem); // must be in dom
		},
		getAttachmentInfo: function(number) {
			return this.parentController.getAttachmentInfo(number);
		},
		expand: function() {
			this.isCollapsed = false;
			this._commentHeader.title = this.messages.titleCommentCollapse;
			if(this.isNew) {
				this.showNewComment();
//				this.commentTextArea.domNode.style.display="block";
//				this._workItemComment.className = "workItemNewComment";
			} else {
				this._commentBodySPAN.style.display="block";
				this._workItemComment.className = "workItemComment";
			}
		},
		collapse: function() {
			this.isCollapsed = true;
			this._commentHeader.title = this.messages.titleCommentExpand;
			if(this.isNew) {
				this._workItemComment.className = "workItemCommentNewCollapsed";
				this.commentTextArea.domNode.style.display = "none";
			} else {
				this._workItemComment.className = "workItemCommentCollapsed";
				this._commentBodySPAN.style.display="none";
			}
		},
		handleClick: function(e) {
			if(this.isNew) return;
			if(e.target.tagName=="A") return;
			if(this.isCollapsed) {
				this.expand();
			} else {
				this.collapse();
			}
		},
		_handleDirtyingEvent: function(e) {

		},
		handleMouseOver: function(e) {
			if(this.isNew) return;
			this._commentHeader.className  = "workItemCommentHeaderMouseOver";
		},
		handleMouseOut: function(e) {
			if(this.isNew) return;
			this._commentHeader.className = "workItemCommentHeader";	
		},
		getTextValue: function() {
			return this.commentTextArea.getValue();
		},
		setTextValue: function(value) {
//			this.showNewComment();
			this.commentTextArea.setValue(value);
			this.newCommentValue = "";
		},
		clearTextValue: function() {
//			this.showNewComment();
			this.commentTextArea.setValue("");
			this.newCommentValue = ""; 
		},
		showNewComment: function() {
			this._workItemComment.className = "workItemNewComment";
			this._commentHeader.style.display="";
			this._commentHeader.className = "workItemNewCommentHeader";
			this._newCommentArea.style.display="";
			this.commentTextArea.domNode.style.display="";
//		"<span class=\"commentNumber\">&nbsp;</span><span class=\"commentDetails\"><B>New Comment</B></SPAN" ;

//			"<span class=\"newCommentHeader\"><B>New Comment</B></SPAN" ;

			this._commentHeaderSPAN = document.createElement("SPAN");
			this._commentHeaderSPAN.className="newCommentHeader";
			var b= document.createElement("B");
			b.appendChild(document.createTextNode(this.messages.buttonNewComment));
			this._commentHeaderSPAN.appendChild(b);
				
			// force the browser to show the save button
			try {	var wasDisabled = this._saveCmdBottom.disabled;
					this._saveCmdBottom.disabled=false;
					this._saveCmdBottom.focus();
					this._saveCmdBottom.disabled=wasDisabled;
					} catch (e) { /* OK, not visible */};
			try {this.commentTextArea.focus();} catch (e) { /* OK, not visible */};
			
			// Watch the new comment value for changes
			this.newCommentValue = this.commentTextArea.getValue();
			this._stopNewCommentInterval();
			this.newCommentInterval = setInterval(dojo.hitch(this, this._checkForNewCommentChanges), 200);
			
		},
		hideNewComment: function() {
			this._commentHeader.style.display="none";
			this._newCommentArea.style.display="none";
			this.commentTextArea.domNode.style.display="none";
			this._stopNewCommentInterval();
			
		},
		_checkForNewCommentChanges: function() {
			if(this.newCommentValue != this.commentTextArea.getValue()) {
				this._stopNewCommentInterval();
				if(this.parentWidget){
					var e = {
						srcElement: this.commentTextArea
					};
					this.parentWidget._handleChange(e);
				}
			}
		},
		_stopNewCommentInterval: function() {
			if(this.newCommentInterval) {
				window.clearInterval(this.newCommentInterval);	
			}
		}
	}
);

})();

