/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.SimpleInputWidget");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("dojox.data.dom");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");

(function() {

var WidgetUtil  = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var EditorUtil 	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.layout.SimpleInputWidget",
	[dijit._Widget, dijit._Templated],
	{
		templateString: '<span class="SimpleInputWidget NarrowWidget">' +
						'<img dojoAttachPoint="_image" class="PreFieldImage" >' +
						'<span dojoAttachPoint="_inputSet" style="display: inline">'+ 
						   '<input dojoAttachPoint="_input" dojoAttachEvent="onkeyup: _handleTextChange" class="OffsetInput WidgetInput TextInput"></input>'+
						   '&nbsp;<span class="UnitTextValue" dojoAttachPoint="_unitSpan1"></span>'+
						'</span>' +
						'<span dojoAttachPoint="_labelSet" style="display: none">' +
						   '<span dojoAttachPoint="_readOnlyValue" class="ReadOnlyTextValue"></span>&nbsp;'+
						    '<span class="ReadOnlyTextValue" dojoAttachPoint="_unitSpan2"></span>' +
						'</span>' +
						'</span>',
     	constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
		},
		postCreate: function() {
			
			this.setUnits(this.units);

			this._inputSet.style.display = "inline";
			this._labelSet.style.display = "none";
			
			WidgetUtil.setSpacerImage(this._image);

			var value = WidgetUtil.getFirstValue(this.values);
			this._input.value = value;
			this._setReadOnlyValue(value);
			
			if(this._modifiers && this._modifiers.onCreate) {
					this.parentWidget[this._modifiers.onCreate](this, value);
			}
			
			if(this._modifiers && this._modifiers.onValueChanged) {
					this.parentWidget[this._modifiers.onValueChanged](this , value, /* isInitialization */ true);
				}
				
			this.isChanged = false;
			if(this.inputWidth) {
				this._input.style.width = this.inputWidth;
			}
			if (this.parentController.pageController.isGuest()) {
				dojo.addClass(this._input, "GuestField");
			}
		},
		setReadOnly: function(isReadOnly) {
			if (isReadOnly) { // was mouse out
				if (this._isChanged) {
					this.setReadOnly(false);
					return;
				}
				this._inputSet.style.display = "none";
				this._labelSet.style.display = "inline";
			}
			else {
				if(this._modifiers && this._modifiers.isReadOnly) return;
				this._inputSet.style.display = "inline";
				this._labelSet.style.display = "none";
			}
		},
		setUnits: function(units){
			this.units = units;
			dojox.data.dom.removeChildren(this._unitSpan1);
			dojox.data.dom.removeChildren(this._unitSpan2);
			if(units) {
				var text1 = document.createTextNode(this.units);
				this._unitSpan1.appendChild(text1);
				var text2 = document.createTextNode(this.units);
				this._unitSpan2.appendChild(text2);
			}
		},
		_setReadOnlyValue: function(value){
			if(value==null) {
				value = this.messages.labelDefaultNullValue;
				dojox.data.dom.removeChildren(this._unitSpan1);
				dojox.data.dom.removeChildren(this._unitSpan2);
			} else {
				if (value.length == 0) {
					value = this.messages.labelDefaultEmptyValue;
//					dojox.data.dom.removeChildren(this._unitSpan1);
					dojox.data.dom.removeChildren(this._unitSpan2);
				} else {
					this.setUnits(this.units);
				}		
			}

			dojox.data.dom.removeChildren(this._readOnlyValue)
			this._readOnlyValue.appendChild(document.createTextNode(value));
		},
		_handleTextChange: function(e) {
			this.isChanged=true;
			this.parentWidget._handleDirtyingEvent(e);
			this._setReadOnlyValue(this._input.value);
			
			if(this._modifiers && this._modifiers.onValueChanged && this.parentWidget && this.parentWidget[this._modifiers.onValueChanged]) {
				this.parentWidget[this._modifiers.onValueChanged](this , this._input.value, /* isInitialization */ false);
			}
			
		},
		getFormElementValues: function(formElements) {
			if (this.isChanged) {
				formElements[this.attributeId] = this._input.value;
			}
		},
		setFormElementValues: function(retainedState){
		 	if(retainedState[this.attributeId]) {
				if(this._input.value != retainedState[this.attributeId]) {
					this._input.value = retainedState[this.attributeId];
					this._setReadOnlyValue(this._input.value);
					this.isChanged = true;
					if(this._modifiers && this._modifiers.onValueChanged && this.parentWidget && this.parentWidget[this._modifiers.onValueChanged]) {
						this.parentWidget[this._modifiers.onValueChanged](this , this._input.value);
					}
				}
			}
		 },
	     getValue: function() {
			return this._input.value;
		 }
	}
);

})();
