/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.plan.IterationPlanPage"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.require("dojox.data.dom"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.util"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.ImageClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.PlanExtensionRegistry"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.Assert"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.TabAttachedPage"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.TabPlannedItems"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.TabContributedPage"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.search.IterationPlanBanner"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.widgets.ProgressBar"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.common.ProgressInformation"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("dojo.i18n"); //$NON-NLS-1$

(function() {

var Assert					= com.ibm.team.apt.web.ui.internal.common.Assert;

var PlanningUtil			= com.ibm.team.apt.web.client.util;
var ImageClient				= com.ibm.team.apt.web.client.internal.ImageClient;
var PlanExtensionRegistry	= com.ibm.team.apt.web.client.internal.PlanExtensionRegistry;
var ProgressBar				= com.ibm.team.apt.web.ui.internal.view.widgets.ProgressBar;
var TabPlannedItems			= com.ibm.team.apt.web.ui.internal.view.plan.TabPlannedItems;
var TabAttachedPage			= com.ibm.team.apt.web.ui.internal.view.plan.TabAttachedPage;
var TabContributedPage		= com.ibm.team.apt.web.ui.internal.view.plan.TabContributedPage;
var IterationPlanBanner		= com.ibm.team.apt.web.ui.internal.view.search.IterationPlanBanner;
var ProgressInformation		= com.ibm.team.apt.web.ui.internal.common.ProgressInformation;

var TID_PREFIX				= "com_ibm_team_apt_web_ui_internal_view_plan_IterationPlanPage-"; //$NON-NLS-1$

var replaceChildren			= dojox.data.dom.replaceChildren;
var removeChildren			= dojox.data.dom.removeChildren;

var bind= dojo.string.substitute;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.IterationPlanPage", //$NON-NLS-1$
			 [dijit._Widget, dijit._Templated], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_iterationPlanDTO: null,
	_referencedItems: null,
	_serviceCallController: null,

	_metaNav: null,
	_typeIcon: null,
	_planName: null,
	_errorSpan: null,
	_infoBarTeamArea: null,
	_infoBarIteration: null,
	_infoBarClosedItems: null,
	_infoBarOpenItems: null,
	
	_progressBar: null,
	_progressHours: null,
	_progressQuality: null,
	
	_tabLinkContainer: null,
	_tabContent: null,
	
	_labelDefaultTitle : MSGS.ITERATIONPLAN_DEFAULTTITLE,
	_labelTeamArea : MSGS.ITERATIONPLAN_TEAMAREA,
	_labelIteration : MSGS.ITERATIONPLAN_ITERATION,
	
	constructor: function() {
		this._tabWidgets= new Object();
		this._currentTabId= null;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------	

	getPlanId: function() {
		return this._iterationPlanDTO.itemId;
	},

	getTitle: function() {
		return bind(MSGS.ITERATIONPLAN_PAGETITLE, [this.getDecoratedPlanLabel()]);
	},

	getDecoratedPlanLabel: function() {
		return bind(MSGS.ITERATIONPLAN_PLAN_LABEL, [this._iterationPlanDTO.label, this._getIteration().label]);
	},

	getPlanIcon: function() {
		return ImageClient.getImageUrl(this._iterationPlanDTO.iconURL);
	},
	
	setViewOptions: function(viewOptions, searchResults) {
		var pageId= viewOptions["page"]; //$NON-NLS-1$
		Assert.isLegal(pageId != null);
		
		if (this._setCurrentTab(pageId)) {
			var currentTab= this._getCurrentTab();
			Assert.isNotNull(currentTab);
			
			currentTab.setViewOptions(viewOptions);
		}
		
		if (searchResults != null) {
			var metaNav= new IterationPlanBanner(IterationPlanBanner.constructorArgs(this._iterationPlanDTO, searchResults, viewOptions));
			replaceChildren(this._metaNav, metaNav.domNode);
		} else {
			removeChildren(this._metaNav);
		}
	},

	getViewOptions: function() {
		var currentTab= this._getCurrentTab();
		
		if (currentTab == null)
			return {};
		
		return dojo.mixin({ page: currentTab.getId() }, currentTab.getViewOptions());
	},

	// ---- widget implementation ------------------------------------------------------------------------------------------

	templatePath: dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/plan/templates/IterationPlanPage.html"), //$NON-NLS-1$ //$NON-NLS-2$

	postCreate: function() {

		this._typeIcon.src= this.getPlanIcon();	
		replaceChildren(this._planName, document.createTextNode(this._iterationPlanDTO.label));
		
		var teamArea= this._getTeamArea()
		var iteration= this._getIteration()
		
		var progressInfo= new ProgressInformation(this._iterationPlanDTO.teamLoad);
		
		this._infoBarTeamArea.appendChild(document.createTextNode(teamArea.label));
		
		var iterationLabel= iteration.label;
		
		var startDateLabel= PlanningUtil.formatDate(iteration.startDate, null, "short"); //$NON-NLS-1$
		var endDateLabel= PlanningUtil.formatDate(iteration.endDate, null, "short"); //$NON-NLS-1$
		if (startDateLabel && endDateLabel) {
			iterationLabel= bind(MSGS.ITERATIONPLAN_ITERATIONLABEL_WITH_DATES, [iteration.label, startDateLabel, endDateLabel]);
		}

		this._infoBarIteration.appendChild(document.createTextNode(iterationLabel));

		this._infoBarClosedItems.appendChild(document.createTextNode(bind(MSGS.ITERATIONPLAN_CLOSEDITEMS, [progressInfo.getCloseCount()])));
		this._infoBarOpenItems.appendChild(document.createTextNode(bind(MSGS.ITERATIONPLAN_OPENITEMS, [progressInfo.getOpenCount()])));

		var constructorArgs= ProgressBar.constructorArgs(progressInfo, "100%", "1.5em"); //$NON-NLS-1$ //$NON-NLS-2$
		var progress= new ProgressBar(constructorArgs, this._progressBar);
		var progressLabelProvider= progress.getLabelProvider();

		this._progressHours.appendChild( progressLabelProvider.getHoursTextNode());
		this._progressQuality.appendChild( progressLabelProvider.getEstimateTextNode());

		var planTabs= [];

		var constructorArgs= TabPlannedItems.constructorArgs(this, null, this._serviceCallController);
		planTabs.push(new TabPlannedItems(constructorArgs));
		
		for (var i = 0; i < this._iterationPlanDTO.attachedPages.length; i++) {
			var attachedPage = this._iterationPlanDTO.attachedPages[i];

			var constructorArgs= TabAttachedPage.constructorArgs(this, attachedPage, null, this._serviceCallController);
			planTabs.push(new TabAttachedPage(constructorArgs));
		}
		
		dojo.forEach(PlanExtensionRegistry.getContributedTabs(), dojo.hitch(this, function(tabDescription) {
			if (tabDescription.id != "com.ibm.team.reports.web.charts" || this._iterationPlanDTO.showChartPage) { //$NON-NLS-1$
				var constructorArgs= TabContributedPage.constructorArgs(this, tabDescription.id, tabDescription.name, tabDescription.widgetId, this._serviceCallController);
				planTabs.push(new TabContributedPage(constructorArgs));
			}
		}));

		this._sortTabs(planTabs);
		
		for (var i= 0; i < planTabs.length; i++) {
			this._addTab(planTabs[i], i == 0);
		}
	},
	
	destroy: function() {
		for (var tabName in this._tabWidgets) {
			this._tabWidgets[tabName].destroy();
		}
		this._tabWidgets= null;

		this.inherited(arguments);
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	_getPlanViewUrl: function(viewOptions) {
		return PlanningUtil.getPlanViewURI(this._iterationPlanDTO.itemId, viewOptions || {});
	},

	_getIteration: function() {
		return this._iterationPlanDTO.getIteration();
	},

	_getTeamArea: function() {
		return this._iterationPlanDTO.getTeamArea();
	},

	_getItem: function(type, id) {
		return this._referencedItems.getItem(type, id);
	},

	// ---- tabs implementation --------------------------------------------------------------------------------------------

	_addTab: function(tabWidget, isFirst) {
		var tabId= tabWidget.getId();
		
		var link = document.createElement("a"); //$NON-NLS-1$
		dojo.addClass(link,"tab"); //$NON-NLS-1$
		link.href = this._getPlanViewUrl({ page : tabId });
		link.id= TID_PREFIX + tabId;

		
		var innerWrapper = document.createElement("div"); //$NON-NLS-1$
		innerWrapper.className = "innerWrapper"; //$NON-NLS-1$
		
		var topWrapper = document.createElement("div"); //$NON-NLS-1$
		topWrapper.className = "topWrapper"; //$NON-NLS-1$
		
		var topBorderDiv = document.createElement("div"); //$NON-NLS-1$
		topBorderDiv.className = "topBorder"; //$NON-NLS-1$
		topWrapper.appendChild(topBorderDiv);
		
		var cornerDiv = document.createElement("div"); //$NON-NLS-1$
		cornerDiv.className = "corners"; //$NON-NLS-1$
		topWrapper.appendChild(cornerDiv);
								
		link.appendChild(topWrapper);

		
		var titleDiv = document.createElement("div"); //$NON-NLS-1$
		titleDiv.className = "title"; //$NON-NLS-1$
		titleDiv.appendChild(document.createTextNode(tabWidget.getTitle()));
		innerWrapper.appendChild(titleDiv);

		link.appendChild(innerWrapper);
		this._tabLinkContainer.appendChild(link);

		this._tabWidgets[tabId]= tabWidget;
	},
	
	_setCurrentTab: function(tabId) {
		if (this._currentTabId == tabId)
			return;
			
		Assert.isLegal(tabId, "tabId"); //$NON-NLS-1$
		
		var tabWidget= this._tabWidgets[tabId];
		if (!dojo.isObject(tabWidget)) {
			return false;
		}
		
		dojo.query(".Selected", this._tabLinkContainer).removeClass("Selected"); //$NON-NLS-1$ //$NON-NLS-2$
		dojo.query("a#" + TID_PREFIX + tabId, this._tabLinkContainer).addClass("Selected"); //$NON-NLS-1$ //$NON-NLS-2$

		dojox.data.dom.removeChildren(this._tabContent);
		this._tabContent.appendChild(tabWidget.domNode);
		
		tabWidget.widgetAboutToShow();
		this._currentTabId= tabId;
		
		return true;
	},
	
	_getCurrentTab: function() {
		return this._currentTabId != null ? this._tabWidgets[this._currentTabId] : null;
	},
	
	_sortTabs: function(tabWidgets) {
		var computeCategory= function(tab) {
			switch (tab.declaredClass) {
				case "com.ibm.team.apt.web.ui.internal.view.plan.TabAttachedPage": //$NON-NLS-1$
					return tab.getId() == "overview" ? 1 : 4; //$NON-NLS-1$
				case "com.ibm.team.apt.web.ui.internal.view.plan.TabPlannedItems": //$NON-NLS-1$
					return 2;
				case "com.ibm.team.apt.web.ui.internal.view.plan.TabContributedPage": //$NON-NLS-1$
					return 3;
				default:
					return 5;
			}
		}

		tabWidgets.sort(function(a, b) {
			var result= computeCategory(a) - computeCategory(b);
			if (result != 0)
				return result;
			
			return a.getTitle().localeCompare(b.getTitle());
		});
	},

	__sentinel: null // terminates this class definition
});

/**
 * This helper method creates the arguments object for the widget's constructor.
 * @param {LoadIterationPlanResultDTO} loadIterationPlanResultDTO the DTO carrying the IterationPlanDTO and its referenced items 
 * @return {Object} a widget private object to be passed in as a constructor argument
 */
com.ibm.team.apt.web.ui.internal.view.plan.IterationPlanPage.constructorArgs= function(loadIterationPlanResultDTO, serviceCallController) {
	var result= {
		_iterationPlanDTO: loadIterationPlanResultDTO.plan,
		_referencedItems: loadIterationPlanResultDTO._referencedItemStore,
		_serviceCallController: serviceCallController
	};
	
	if (loadIterationPlanResultDTO.initialPage) {
		result["_pageData"]["initial"]= loadIterationPlanResultDTO.initialPage; //$NON-NLS-1$ //$NON-NLS-2$
	}

	return result;
};

})();
