/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.ConditionSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.core.internal.loader");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.VariableControl");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.ErrorLabel");

dojo.require("com.ibm.team.workitem.web.client.internal.query.AttributeExpression");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
(function(){

var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var ServiceResponseHandler= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var QueryClient= com.ibm.team.workitem.web.client.internal.QueryClient;
var VariableControl= com.ibm.team.workitem.web.ui.internal.view.queryeditor.VariableControl;
var loader= net.jazz.ajax.core.internal.loader;
var AttributeExpression= com.ibm.team.workitem.web.client.internal.query.AttributeExpression;
var ErrorLabel= com.ibm.team.workitem.web.ui.internal.view.queryeditor.ErrorLabel;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.ConditionSection", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/templates/ConditionSection.html"),
	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		this.textLoading = this.messages.busyLoading;
		this.attributeId= args.attributeId; // attribute identifier
		this.parent= args.parent; // TermSection
		this.expression= args.expression; // AttributeExpressionDTO
		
		this._variableControls= new Array();
		this._operator= null;
	},
	
	postCreate: function() {
		this._errorLabel= new ErrorLabel({ errorElement: this._errorDiv });		
	},
	destroy: function() {
			try {
				if(this._errorLabel) this._errorLabel.destroy();
				if(this._presentationWidget && this._presentationWidget.destroy) this._presentationWidget.destroy();
				
				dojo.forEach(this._variableControls,  function(control) {
					if(control.destroy) control.destroy();
				});
				
				this.inherited(arguments);
			} catch (e) {
				
			}
	},
	setSectionSite: function(site) {
		this._site= site;
		this._loadPresentation();
	},
	
	/* AttributeExpression */ getExpression: function() {
		
		var variables= this._getVariables();
		var values= this._getValues();
		
		if ((values && values.length > 0) || (variables && variables.length > 0) || (this._operator && this._operator.arity == 0)) {
			var expression= new AttributeExpression(this.attributeId, this._operator.id);
			expression.setVariables(variables);
			expression.setValues(values);
			return expression;
		}
		return null;
	},
	
	onClose: function() {
		this.parent.deregister(this);
		return true;
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Internals
	
	_loadPresentation: function() {
		var srh= new ServiceResponseHandler(this, "_createWidget", "_handlePresentationError");
		QueryClient.getPresentation(srh, this.attributeId);
	},
	
	_createWidget: function(/* PresentationDTO */ presentation) {
		
		if (!presentation) {
			this._hideLoadingMessage();
			return;
		}
			
		var site= this._site;
		var operator= this.expression ? this.expression.operator : presentation.operators[0];
		this._setOperator(operator);
		
		site.setTitle(presentation.attribute.label);
		site.setupMenu("label", dojo.hitch(this, this._handleSetOperator));
		site.setMenuItems(presentation.operators);
		site.setSelectedMenuItem(operator);
		
		this._createVariableControls(presentation.variables, this.expression ? this.expression.variables : []);
		
		this._loadWidget(presentation);
	},
	
	_loadWidget: function(/* PresentationDTO */ presentation) {
		
		var widgetClass= presentation.widgetClass;
		
		if (dojo.exists(widgetClass)) {
			this._showWidget(presentation);
			return;
		}
		
		var self= this;
		loader.load_async(widgetClass, function() {
			try {
				self._showWidget(presentation);
			}
			catch (e) {
				self._handleWidgetError(e);
			}
		});
	},
	
	_showWidget: function(/* PresentationDTO */ presentation) {
		
		this._hideLoadingMessage();
		
		var selectedValues= this.expression ? this.expression.values : [];
		var selectedVariables= this.expression ? this.expression.variables : [];
		var errorCallback= dojo.hitch(this, this._showError);
		
		var ctor= dojo.getObject(presentation.widgetClass);
		var widget= new ctor({ presentation: presentation, selection: selectedValues, variables: selectedVariables, errorCallback: errorCallback });
		this._conditionContent.appendChild(widget.domNode);
		this._presentationWidget= widget;
	},
	
	_createVariableControls: function(/* VariableDTO[] */ variables, /* VariableDTO[] */ used) {
		
		if (!variables)
			variables= [];
			
		if (!used)
			used= [];
			
		if (variables.length == 0) {
			this._columns.style.display= "none";
			return;
		}
		
		var seen= {};
		var k= 0;
		
		for (var i= 0; i < variables.length; i++) {
			this._createVariableCheckBox(variables[i], k % 2);
			seen[variables[i].label]= true;
			k++;
		}
		for (var i= 0; i < used.length; i++) {
			if (!seen[used[i].label]) {
				this._createVariableCheckBox(used[i], k % 2);
				k++;
			}
		}
	},
	
	_createVariableCheckBox: function(/* VariableDTO */ variable, column) {
		var variableControl= new VariableControl({ variable: variable });
		var columnDiv= this["_column" + column];
		columnDiv.appendChild(variableControl.domNode);
		this._variableControls.push(variableControl);
		
		if (this._isSelected(variable)) {
			variableControl.setChecked(true);
		}
	},
	
	_isSelected: function(/* VariableDTO */ variable) {
		if (this.expression && this.expression.variables) {
			var selected= this.expression.variables
			for (var i= 0; i < selected.length; i++) {
				if (selected[i].id == variable.id && selected[i].arguments == variable.arguments)
					return true;
			}
		}
		return false;
	},
	
	/* Variable[] */ _getVariables: function() {
		
		var variables= new Array();
		
		var controls= this._variableControls;
		if (controls) {
			for (var i= 0; i < controls.length; i++) {
				if (controls[i].isChecked())
					variables.push(controls[i].getVariable());
			}
		}
		var widgetVariables= this._getWidgetVariables();
		if (widgetVariables) {
			for (var i= 0; i < widgetVariables.length; i++) {
				variables.push(widgetVariables[i]);
			}
		}
			
		return variables;
	},
	
	/* Variable[] */ _getWidgetVariables: function() {
		var widget= this._presentationWidget;
		if (!widget || !widget.getVariables)
			return [];
		return widget.getVariables();
	},
	
	/* String[] */ _getValues: function() {
		var widget= this._presentationWidget;
		if (!widget)
			return [];
		return widget.getValues();
	},
	
	_handleSetOperator: function(operator) {
		this._setOperator(operator);
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
	},
	
	_setOperator: function(operator) {
		this._operator= operator;
	},
	
	_hideLoadingMessage: function() {
		if (this._loadingMessage) {
			this._loadingMessage.parentNode.removeChild(this._loadingMessage);
			this._loadingMessage= null;
		}
	},
	
	_handlePresentationError: function(error) {
		this._showError(this.messages.errorFetchingPresentation, error);
	},
	
	_handleWidgetError: function(error) {
		this._showError(this.messages.errorFetchingWidget, error);
	},
	
	_showError: function(message, error) {
		this._errorLabel.showError(message, error);
	}
	
});
	
})();
