/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.search.SearchResultsPageControl"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.util"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("dojo.i18n"); //$NON-NLS-1$

(function() {

var PlanningUtil		= com.ibm.team.apt.web.client.util;

var bind= dojo.string.substitute;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$ 

dojo.declare("com.ibm.team.apt.web.ui.internal.view.search.SearchResultsPageControl", //$NON-NLS-1$
			 [dijit._Widget, dijit._Templated], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_showHeader: null,
	_searchResults: null,
	_startIndex: 0,
	
	_pageNumberContainer: null,

	// ---- api ------------------------------------------------------------------------------------------------------------	


	// ---- widget implementation ------------------------------------------------------------------------------------------

	templatePath: dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/search/templates/SearchResultsPageControl.html"), //$NON-NLS-1$ //$NON-NLS-2$ 

	postCreate: function() {
		this._createQueryLink();
		this._createPaginationControl();
	},
	
	// ---- implementation -------------------------------------------------------------------------------------------------

	_createQueryLink: function() {
		dojo.addClass(this.domNode, this._style);
		var queryDescriptor= this._searchResults.getQueryDescriptor();
		this._titleContainer.appendChild(document.createTextNode(queryDescriptor.getLabel()));
	},

	_createPaginationControl: function() {
		var queryDescriptor= this._searchResults.getQueryDescriptor();
		
		var startIndex= this._startIndex;
		var resultCount= this._searchResults.getResultCount();
		var pageSize= this._searchResults.getResultPageSize();
		
		if (resultCount > 0) {
			// show previous page text or link
			this._pageNumberContainer.appendChild(this._getPreviousElement(startIndex, resultCount, pageSize, queryDescriptor));
			
			// show count info
			var pageInfoSpan= document.createElement("span"); //$NON-NLS-1$
			
			pageInfoSpan.className= "pageInfoSpan"; //$NON-NLS-1$
			var endIndex = Math.min(startIndex + pageSize, resultCount);
			pageInfoSpan.innerHTML = bind(MSGS.PAGECONTROL_PAGEINFO, [startIndex + 1, endIndex, resultCount]);
			this._pageNumberContainer.appendChild(pageInfoSpan);
			
			// show next page link or text
			this._pageNumberContainer.appendChild(this._getNextElement(startIndex, resultCount, pageSize, queryDescriptor));
		}
	},
	
	_getPreviousElement: function(startIndex, resultCount, pageSize, queryDescriptor) {
		var label= MSGS.PAGECONTROL_PREVIOUS;
		if (startIndex > 0) {
			return this._getLinkElement(PlanningUtil.getPaginatedSearchURI(queryDescriptor, Math.max(startIndex - pageSize, 0)), label);
		}
		return this._getDisabledTextElement(label);
	},
	
	_getNextElement: function(startIndex, resultCount, pageSize, queryDescriptor) {
		var label= MSGS.PAGECONTROL_NEXT;
		if (startIndex + pageSize < resultCount) {
			return this._getLinkElement(PlanningUtil.getPaginatedSearchURI(queryDescriptor, startIndex + pageSize), label);
		}
		return this._getDisabledTextElement(label);
	},
	
	_getDisabledTextElement: function(str) {
		var span= document.createElement("span"); //$NON-NLS-1$
		span.className= "disabledText"; //$NON-NLS-1$
		span.appendChild(document.createTextNode(str));
		return span;
	},
	
	_getLinkElement: function(href, text) {
		var linkElement = document.createElement("a"); //$NON-NLS-1$
		linkElement.href = href;
		linkElement.appendChild(document.createTextNode(text));
		return linkElement;
	},

	__sentinel: null // terminates this class definition
});

/**
 * This helper method creates the arguments object for the widget's constructor.
 * @param {SearchResults} searchResults The search results displayed 
 * @return {Object} a widget private object to be passed in as a constructor argument
 */
com.ibm.team.apt.web.ui.internal.view.search.SearchResultsPageControl.constructorArgs= function(searchResults, startIndex, style) {
	return { 
		_style: style,
		_searchResults: searchResults,
		_startIndex: startIndex
	 };
};

})();
