/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.widgets.ProgressBar"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.require("dojox.data.dom"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.util"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.ProgressLabelProvider"); //$NON-NLS-1$

(function() {

var PlanningUtil		= com.ibm.team.apt.web.client.util;

var replaceChildren			= dojox.data.dom.replaceChildren;

dojo.declare("com.ibm.team.apt.web.ui.internal.view.widgets.ProgressBar", //$NON-NLS-1$
			 [dijit._Widget, dijit._Templated], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_progressInfo: null,
	_labelProvider: null,
	_width: null,
	_height: null,
	
	_progressBar: null,
	_barContainer: null,
	_noWorkEstimatedLabel: null,

	// ---- api ------------------------------------------------------------------------------------------------------------	

	getLabelProvider: function() {
		return this._labelProvider;
	},

	// ---- widget implementation ------------------------------------------------------------------------------------------

	templatePath: dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/widgets/templates/ProgressBar.html"), //$NON-NLS-1$ //$NON-NLS-2$

	postCreate: function() {
		dojo.style(this._progressBar, "width", this.width); //$NON-NLS-1$
		dojo.style(this._progressBar, "font-size", this.height); //$NON-NLS-1$
		
		
		var message= this._labelProvider.getMessage();

		if (message != null) {
			dojo.style(this._barContainer, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
			dojox.data.dom.replaceChildren(this._noWorkEstimatedLabel, document.createTextNode(message));
		} else {
			dojo.style(this._noWorkEstimatedLabel, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
			
			this._applyStyle(".barContainer .notEstimated", "height", 1 - (this._progressInfo.getQualityOfPlanning() / 100.0)); //$NON-NLS-1$ //$NON-NLS-2$
			this._applyStyle(".barContainer .estimated", "height", (this._progressInfo.getQualityOfPlanning() / 100.0)); //$NON-NLS-1$ //$NON-NLS-2$ 

			var workHourProgress= this._progressInfo.getWorkHourProgress();

			var completed= 0;
			var delta= 0;

			if (this._progressInfo.isAllWorkDone()) {
				var completed= 1;
			} else if (this._progressInfo.isRealTimeSpecified()) {
				var realTimeProgress= this._progressInfo.getRealTimeProgress();
				completed= Math.min(realTimeProgress, workHourProgress);
				delta= workHourProgress - realTimeProgress;
			} else {
				// no end date on the iteration
				completed= workHourProgress;
			}

			this._applyStyle(".barContainer .completed", "width", completed); //$NON-NLS-1$ //$NON-NLS-2$
			this._applyStyle(".barContainer .difference", "left", completed); //$NON-NLS-1$ //$NON-NLS-2$
			this._applyStyle(".barContainer .difference", "width", Math.abs(delta)); //$NON-NLS-1$ //$NON-NLS-2$
			this._applyStyle(".barContainer .open", "width", 1 - completed - Math.abs(delta)); //$NON-NLS-1$ //$NON-NLS-2$
			
			dojo.query(".barContainer .difference", this._progressBar).addClass(delta < 0 ? "behind" : (delta > 0 ? "ahead" : "none")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ 
		}
		
		var tooltipContentSpan= document.createElement("span"); //$NON-NLS-1$
		tooltipContentSpan.innerHTML= this._labelProvider.getTooltipContent();
		
		new net.jazz.ajax.ui.internal.HoverPopup({
			connectNode: dojo.query("div", this._progressBar) //$NON-NLS-1$
		}, tooltipContentSpan);

	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	_applyStyle: function(query, style, value) {
		dojo.query(query, this._progressBar).style(style, Math.round(value * 100) + "%"); //$NON-NLS-1$
	},

	__sentinel: null // terminates this class definition
});

/**
 * This helper method creates the arguments object for the widget's constructor.
 * @return {Object} a widget private object to be passed in as a constructor argument
 */
com.ibm.team.apt.web.ui.internal.view.widgets.ProgressBar.constructorArgs= function(progressInfo, width, height) {
	return {
		_progressInfo: progressInfo,
		_labelProvider: new com.ibm.team.apt.web.ui.internal.common.ProgressLabelProvider(progressInfo),
		width: width || "100%", //$NON-NLS-1$
		height: height || "1em" //$NON-NLS-1$
	};
};

})();
